/***************************************************************************
 * digimon.c
 * Note: Images are oriented with pixel (0,0) in the top left corner.
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <GL/glut.h>
#include <limits.h>    /* This is where SHRT_MAX is defined */
#include <time.h>

#define TRUE  1
#define FALSE 0

#ifndef DBL_MAX
#define DBL_MAX         1.7976931348623157E+308
#endif
//#define PIE 3.14159265358979323846
#define PIE 3.141592653589
#define TWO_PIE (2*PIE)
//6.2831853072
#define MAX_NUM_CLASSES 10
#define HIST_DIM 256
#define FEATURE_SIZE 128
#define MAX_SIGNAL_SIZE 65536
#define X_AXIS 1
#define Y_AXIS 2
#define SINE 1
#define COSINE 2
#define GRY 0
#define RED 1
#define GRN 2
#define BLU 3
#define ABS(a) ((a)>(-a)?(a):(-a))
#define DBL_TOL .0000001


#define MAX_NUM_POINTS 100000
#define MAX_NUM_SEEDS 30
#define WAVE_SIZE 256
#define WAVE_MOD 0xff
#define BOUNDARY_PIXEL 255
#define SHAPE_REP_SIZE 64
#define NUM_GROUPS 4
class Image;
class Complex;
class Cutter;

//GLOBAL VARAIBLES
int cut_xres = 1, cut_yres = 1;
Image *cut_image = NULL;
Cutter *cutter = NULL;
int g_delay;
int button_down = FALSE;


//Function prototypes
void display();
void getNeighbor4(int x, int y, int *nx, int *ny, int id);
void getNeighbor8(int x, int y, int *nx, int *ny, int id);
void getColorFromID(int id, unsigned char *r,  unsigned char *g,  unsigned char *b );
void convolve1D(double *s1, int size1,
                double *s2, int size2,
                double *s3, int *size3);
Image* convolve2D(Image *img1, Image *img2);
Image* sobelFilter(Image *img2);

void medianFilter1D(int range, double *s1, int size1, double *s2, int *size2);
void enhanceEdge1D (double *s1, int size1,
                    double *s2, int size2,
                    double *s3, int *size3);
void slowSort(double *a, int size);
void threshold(double *a, int size, double val, double bot, double top);
void threshBlock2D(double *a, int w, int h, int range, double val, double bot, double top, double *b);
void FourierTransform1D(double *f, int size, Complex *F);
void invFourierTransform1D(Complex *F, int size, double *f);
void FFT1D(double *f, int size, Complex *F);
void FFT1Drec(Complex *F, int size, int mid, int dir);
void FFT2D(double *f, int size, Complex *F);
void invFFT2D(Complex *F, int size, double *f);
void invFFT1D(Complex *F, int size, double *f);
int  findPowerOf2(int size);

void createSinusoid(int size, double w, double *s, double mag, int flag);
void createSinusoid2D(int size, double freq, double *s, double mag, int axis, int flag);
void createSobel(int size, double *sobel);
void plot1D(double *a, int size, char *plot_name, char *filename);
void plotComplex1D(Complex *a, int size, char *plot_name, char *filename);

//Complex Number stuff
Complex add(Complex c1, Complex c2);
Complex sub(Complex c1, Complex c2);
Complex mult(Complex c1, Complex c2);
Complex div(Complex c1, Complex c2);
void add(double *s1, int size1, double *s2, int size2, double *s3, int *size3);
void mult(double *s1, int size1, double *s2, int size2, double *s3, int *size3);
void mult(Complex *s1, int size1, Complex *s2, int size2, Complex *s3, int *size3);
void div(Complex *s1, int size1, Complex *s2, int size2, Complex *s3, int *size3);

//More functions
int  boxThreshold(Image *im, Image *out_im);
void copyDouble(double *dest, double *src, int size);
void dilate(double *in, int w, int h, double *out, double *filter, int size, double tol);
void erode(double *in, int w, int h, double *out, double *filter, int size, double tol);
double getAverage(double *s, int size, double zero_value, double tol);
double getPercentValue(double *s, int size, double val, double tol);
double getDiff(double *s1, double *s2, int size);
double getAveDiff(double *s1, double *s2, int size);
void invert(Complex *s1, int size1);
void rotate2D(double *s, int n, double *d, double theta);
void swapMagPhase(Complex *c1, Complex *c2, int size);
void removeInterference(Complex *D_Data, int size);
void enhanceEdge2D (Complex *c1, int size1,
                    Complex *c2, int size2,
                    Complex *c3, int *size3);
void medianFilter2D(int range, double *s1, int w, int h, double *s2);
void normalize(double *d, int size);
void writeUB(double *s, int size, char *filename);
void butterFilter(Complex *c, int size, double cut_off, double n);
void filterFourier2D(Complex *c, int size, int begin, int end);


//Other Functions
void fwrite_long(unsigned long number, FILE *stream);
void fwrite_short(unsigned short number, FILE *stream);
void write_wav(char *filename, double *sample, unsigned long sample_len, unsigned long freq);
double *create_wav(int sample_len, double freq_scale, double noise_frac);


//Classes
//Classes
//Classes
//Classes
class Complex{
  public:
    double r, i;

    double getMag(){ return sqrt(r*r + i*i); };
    double getPhase(){
      if (i < DBL_TOL || r < DBL_TOL){
        return atan2(0.0, 0.0);
      }
      return atan2(i, r);
    };
};

class Image{

  public:
    unsigned char *data;
    int height, width;
    int type, max;

    Image();
    Image(Image *from_img);
    Image(int w, int h, int type);
    Image(double *signal, int size);
    ~Image();
    unsigned char colorShapes(unsigned char tol, int min_area);
    void copy(Image *img);
    void createInfoImage(int num_shapes);
    void clear(unsigned char val);
    void drawLine(int x0, int y0, int x1, int y1, unsigned char r, unsigned char g, unsigned char b);
    void diff(Image *img);
    void dilate(int size);
    void erode(int size);
    void predCode1(int flag);
    void predCode2(int flag);
    void equalize();
    void flip();
    void getCenterOfGravity(int *x, int *y, unsigned char tol);
    int markCenterOfGravityColor(int *cx, int *cy, unsigned char i);
    double markOrientation(int i, int cy, int cy);
    Image* getHistImage(int flag);
    Image* getEqualizeTableImage();
    void getPDF(double *pdf, int n, double *max_val);
    void getShapeColor(unsigned char *r, unsigned char *g, unsigned char *b, unsigned char *br, unsigned char *bg, unsigned char *bb, unsigned char tol);
    double *getDouble(double offset);

    void getItemBounds(unsigned char id, int *bot_x, int *bot_y, int *top_x, int *top_y);
    unsigned char getBoundAverage(int bot_x, int bot_y, int top_x, int top_y);
    void unionThreshRegion(Image *img, int bot_x, int bot_y, int top_x, int top_y, int range, unsigned char t);

    void invert();
    void makePowerOf2(int new_size);
    void makeColor();
    void makeGray();
    void pad(int new_width, int new_height);
    int read(char *filename);
    void rotate(double deg);
    void makeSinusoid(int size, double freq, double mag, int axis);
    void makeMag(Complex *D_Data, int size);
    void makePhase(Complex *D_Data, int size);
    void mask(Image *m);
    void signalToImage2D(double *d_data, int w, int h);
    void swapQuads();
    void threshold(unsigned char val);
    int write(char *filename);

    void seedFill(int x, int y,
                  unsigned char r, unsigned char g, unsigned char b,
                  unsigned char f_r, unsigned char f_g, unsigned char f_b, 
                  unsigned char tol, int *area);
  private:
};

//Cutter stuff
class GraphNode{
  /*
   * 0,0   ->   width
   *
   * |
   * v
   * height
   *
   * Parent specification
   *  3 2 1
   *  4 - 0
   *  5 6 7
   */
  public:
    unsigned int cost;
    unsigned char parent;
    int total_cost;
    char expanded;
    int x, y;
    GraphNode *next;
    GraphNode(){
      x = y = 0;
      total_cost = 0;
      cost = 0;
      parent = 8;
      expanded = FALSE;
      next = NULL;
    }
};

class PointList{
  public:
//  int curr_point;
  int id;
  int num_points;
  int num_seeds;
  int cx, cy;
  int x[MAX_NUM_POINTS];
  int y[MAX_NUM_POINTS];
  double r[SHAPE_REP_SIZE];//for shape representation.
  int seed[MAX_NUM_SEEDS];//a list of all the points in the x, y arrays that are
                          //seeds.
  PointList *next;

  PointList(){
    id = 0;
    num_points = 0;
    num_seeds = 0;
    cx = cy = 0;
    next = NULL;
  }
  ~PointList(){
    if (next != NULL) delete next;
  }

  void createShapeRep();
  double getDist(PointList *plist);
  double getDist(double *r2);
  void writeShapeRep(char *prefix);
};

class ContourMaker{
  public:
  PointList *plist_head;
  int num_plist;

  ContourMaker(Image *img){
    Image im2;
    plist_head = NULL;
    createBoundaries(img);
    //drawBoundaries(img);
    matchBoundaries();
    im2.copy(img);
    im2.clear(0);
    drawBoundaries(&im2);
    im2.write("out_groups.ppm");
    num_plist = 0;
  }
  ~ContourMaker(){
    if (NULL != plist_head) delete plist_head;
  }

  //TOBECONT
  void createBoundaries(Image *img);
  void matchBoundaries();
  void drawBoundaries(Image *img, PointList *plist);
  void drawBoundaries(Image *img);//draws the boundaries onto this image
                                  //the color of the boundary will be the
                                  //group color.

};

class Cutter{
  public:
  GraphNode *graph;
  GraphNode* wavefront[WAVE_SIZE];
  PointList plist;
  int best_node;
  int wave_done;
  int height, width;
  int started;


  Cutter(){ started = FALSE; height = width = 0; best_node = 0; wave_done = 0;}
  Cutter(Image *img);
  ~Cutter();

  void addSeedPoint();
  void addToWaveFront(int x, int y);
  void clearGraph();
  void deleteWaveFront();
  void dumpCostGraph();
  void dumpPaths();
  void dumpWaveFront();
  int  findInWaveFront(int x, int y);
  int getCost(int x1, int y1, int x2, int y2);
  void initCostGraph(Image *img);//initializes all the costs for the nodes in the graph;
  void initWaveFront(int seed_x, int seed_y);
  void removeSeedPoint();
  void removeFromWaveFront(int x, int y);
  void saveBoundary(int x, int y);
  int  updateWaveFront();
  void updatePointList(int x, int y);//add points to connect from last seed to
                                     //this point in the graph

};




//POINTLIST
//POINTLIST
//POINTLIST
//POINTLIST

//Create the shape rep for each point list.
void PointList::createShapeRep(){
  double d_j, inc;
  int i, j;
  double max;
  int dx, dy;
  char fname[255];
  //Use the R-S representation
  //Radius as function of arc length.
  //TOBECONT
  if (num_points < SHAPE_REP_SIZE){
    printf("Not enough points for my to be a valid shape for me.\n");
    return;
  }
  inc = (double)num_points/(double)SHAPE_REP_SIZE;
  max = 0.0;
  for (i=0; i<SHAPE_REP_SIZE; i++){
    d_j = (double)i * inc;
    j = (int) d_j;
//    printf(" (i: %d, j: %d) ",i, j);
    dx = x[j] - cx;
    dy = y[j] - cy;
    r[i] = sqrt(dx*dx + dy*dy);
    if (max < r[i]) max = r[i];
  }
//  printf("num_points: %d\n", num_points);

  //Normalize the shape rep
  max = 1.0/max;
  for (i=0; i<SHAPE_REP_SIZE; i++){
    r[i]*=max;
  }
//  writeShapeRep("out_shape_rep");

}

//Do corrolation to find the best match against the shape
double PointList::getDist(PointList *plist){
  int i, j;
  double min, dist;
  double *r2;

//  if (id == plist->id) return -1; //same shape, but one is inside &&
                                 // the other out
  min = SHAPE_REP_SIZE*SHAPE_REP_SIZE;
  r2 = plist->r;
  for (i=0; i<SHAPE_REP_SIZE; i++){
    dist = 0;
    for (j=0; j<SHAPE_REP_SIZE; j++){
      dist += fabs( r[j] - r2[(i+j)%SHAPE_REP_SIZE]);
    }
    if (dist < min) min = dist;
  }
  return min;
}

//Compare just with a double array
double PointList::getDist(double *r2){
  int i, j;
  double min, dist;
                                 
  min = SHAPE_REP_SIZE*SHAPE_REP_SIZE;
  for (i=0; i<SHAPE_REP_SIZE; i++){
    dist = 0;
    for (j=0; j<SHAPE_REP_SIZE; j++){
      dist += fabs( r[j] - r2[(i+j)%SHAPE_REP_SIZE]);
    }
    if (dist < min) min = dist;
  }
  return min;
}

//Write out an image of each shape rep
void PointList::writeShapeRep(char *prefix){
  int i, j;
  char fname[255];
  //TOBECONT
  Image *im;
  im = new Image(SHAPE_REP_SIZE*2, SHAPE_REP_SIZE*2, 6);
  im->clear(0);

  for (i=0; i<SHAPE_REP_SIZE-1; i++){
    j = i + 1;
    im->drawLine(i<<1, SHAPE_REP_SIZE*2 - (int)(r[i]*SHAPE_REP_SIZE*2.0), j<<1, SHAPE_REP_SIZE*2 - (int)(r[j]*SHAPE_REP_SIZE*2.0), 255, 50, 64);
  }
  sprintf(fname, "%s%d.ppm", prefix, num_points);
  im->write(fname);
  delete im;
}





//CONTOURMAKER
//CONTOURMAKER
//CONTOURMAKER
//CONTOURMAKER
//Use the over the shoulder algorithm by Dr. Barrett.
void ContourMaker::createBoundaries(Image *img){
  int y, x, ind, ind2, i, j;
  int num_shapes;
  int height, width;
  unsigned char bb, dir;
  unsigned char *data;
  int nx, ny, currx, curry, done;
  PointList *new_plist;
  int totalx, totaly;
  Image *img_ptr;
  Image im;

  im.copy(img);
  img_ptr = &im;

  img_ptr->makeColor();
  height = img_ptr->height;
  width = img_ptr->width;
  //First color the shapes a unique color
  num_shapes = img_ptr->colorShapes(0, 10);

  data = img_ptr->data;
  //Get the background color
  bb = 0;//data[2];

  //TOBECONT
  //Now find the contour
  for(y=0; y<height; y++){
    for(x=0; x<width-1; x++){
      ind = (x+y*width)*3;
      ind2 = (x+1+y*width)*3;
      //We check the pixel after the current position
      if ( data[ind+2] == bb &&
           data[ind2+2] != bb &&
           data[ind2+2] != BOUNDARY_PIXEL ){
 //       printf("Boundary start: (%d, %d) curr:%d n:%d\n", x, y, (int)data[ind+2], (int) data[ind2 +2]);
        //We have found a shape, so make a contour
        currx = x;
        curry = y;
        dir = 0;
        done = FALSE;
        totalx = totaly = 0;
        new_plist = new PointList();
        new_plist->next = plist_head;
        plist_head = new_plist;
        new_plist->id = data[ind2+2];
        num_plist ++;
        for (j=0; TRUE != done && j<100000; j++){
          getNeighbor4(currx, curry, &nx, &ny, dir);
          //While pixel in front is a region pixel, turn left
          for(i=0;
             (data[(nx + ny*width)*3+2] != bb &&
              data[(nx + ny*width)*3+2] != BOUNDARY_PIXEL) && i<4;
              i++){
            dir++;
            dir &= 3;
            getNeighbor4(currx, curry, &nx, &ny, dir);
          }
          if (4==i){
            printf("ERROR in contourMaker::createBoundaries(). Too many rotations\n");
            return;
          }
          //Mark the current position as a boundary pixel
          data[(currx + curry*width)*3+2] = BOUNDARY_PIXEL;
          data[(currx + curry*width)*3] = 0;
          data[(currx + curry*width)*3+1] = 0;
          new_plist->x[new_plist->num_points] = currx;
          new_plist->y[new_plist->num_points] = curry;
          totalx += currx;
          totaly += curry;
          new_plist->num_points ++;
//          printf("curr: (%d, %d)  nx: (%d, %d)\n", currx, curry, nx, ny);
          //Step
          currx = nx;
          curry = ny;
          if ( currx == x && curry == y ){
            done = TRUE;
          }
          else{
            dir = (dir+3)&3;
          }
       //   if (j%10 == 0) display();  //For debug
        } 
        new_plist->cx = totalx/new_plist->num_points;
        new_plist->cy = totaly/new_plist->num_points;
        new_plist->createShapeRep();
      }
    }
  }
 
  img_ptr->write("out_colored.pgm");
}

void ContourMaker::matchBoundaries(){
  PointList *scan_i = NULL;
  PointList *best_i = NULL;
//  PointList **plist_ptr;
  int i, j, k, best_g;
  double group[NUM_GROUPS][SHAPE_REP_SIZE];
  PointList pl[NUM_GROUPS];
  double c, min, max_cost;

  //TOBECONT
  //Start by finding the most dissimilar shapes
  min = SHAPE_REP_SIZE;
  //Add in the first group to the list
  copyDouble(group[0], plist_head->r, SHAPE_REP_SIZE);
  for (k=1; k<NUM_GROUPS; k++){
    best_i = plist_head;
    max_cost = 0;
    for (scan_i=plist_head->next; NULL != scan_i; scan_i = scan_i->next){
      //compare against existing groups
      c=0;
      for (j=0; j<k; j++){
        c += scan_i->getDist(group[j]);
      }
      if (c > max_cost){
        best_i = scan_i;
        max_cost = c;
      }
    }
    copyDouble(group[k], best_i->r, SHAPE_REP_SIZE);
  }

  //Now assign plist to a group
  for (scan_i=plist_head; NULL != scan_i; scan_i = scan_i->next){
    best_g = 0;
    min = SHAPE_REP_SIZE;
    for (j=0; j<NUM_GROUPS; j++){
      c = scan_i->getDist(group[j]);
      if (c < min){
        min = c;
        best_g = j;
      }
    }
    scan_i->id  = best_g;
  }

  for (k=0; k<NUM_GROUPS; k++){
    copyDouble(pl[k].r, group[k], SHAPE_REP_SIZE);
    pl[k].num_points = k;
//    pl[k].writeShapeRep("out_group_");
  }

  //Now we have a cost graph we can use to match up the shapes with.
//  delete [] cost;
//  delete [] has_hole;
//  delete [] plist_ptr;
}

void ContourMaker::drawBoundaries(Image *img){
  unsigned char *data;
  //int h = img->height;
  int w = img->width;
  PointList *scan_i;
  int *x, *y;
  int i;
  unsigned char r, g, b;

  img->makeColor();
  data = img->data;
  for (scan_i = plist_head; NULL != scan_i; scan_i = scan_i->next){
    getColorFromID(scan_i->id, &r, &g, &b);
    x = scan_i->x;
    y = scan_i->y;
    for (i=0; i<scan_i->num_points; i++){
      data[(x[i] + y[i]*w)*3] = r;
      data[(x[i] + y[i]*w)*3+1] = g;
      data[(x[i] + y[i]*w)*3+2] = b;
    } 
  }
}

void ContourMaker::drawBoundaries(Image *img, PointList *plist){
  unsigned char *data;
  //int h = img->height;
  int w = img->width;
  int *x, *y;
  int i;
  unsigned char r, g, b;
  r = 0; g = 0; b = 255;

  img->makeColor();
  data = img->data;
  x = plist->x;
  y = plist->y;
  for (i=0; i<plist->num_points; i++){
    data[(x[i] + y[i]*w)*3] = r;
    data[(x[i] + y[i]*w)*3+1] = g;
    data[(x[i] + y[i]*w)*3+2] = b;
  } 
}

//CUTTER
//CUTTER
//CUTTER
//CUTTER
Cutter::Cutter(Image *img){
  int i, j;
  //int l, m;

  img->makeGray();
  height = img->height;
  width = img->width;

  graph = new GraphNode[height*width];
  for (i=0; i<WAVE_SIZE; i++){
    wavefront[i] = NULL;
  }

  for (j=0; j<height; j++){
    for (i=0; i<width; i++){
      graph[i + j*width].x = i;
      graph[i + j*width].y = j;
    }
  }

  best_node = 0;
  wave_done = 0;
  started = FALSE;
  initCostGraph(img);
}

Cutter::~Cutter(){
  delete [] graph;
}

void Cutter::addSeedPoint(){
  plist.seed[plist.num_seeds] = plist.num_points-1;
  plist.num_seeds ++;
}

void Cutter::removeSeedPoint(){
  int seed_id;
  
  if (plist.num_seeds <= 0) return;
  plist.num_seeds --;
  if (plist.num_seeds == 0){
    plist.num_points = 0;
    started = FALSE;
  }
  else{
    //Initialize the wave front for the previous seed
    seed_id = plist.seed[plist.num_seeds-1];
    initWaveFront(plist.x[seed_id], plist.y[seed_id]);
    updateWaveFront();
  }
}

void Cutter::removeFromWaveFront(int x, int y){
  int done;
  int loc;
  GraphNode *prev, *curr, *next;

  done = FALSE;
  loc = (x + y*width) & WAVE_MOD;
  for (prev = NULL, curr = wavefront[loc];
       NULL != curr && !done;
       prev = curr, curr = next){
    next = curr->next;
    if (curr->x == x && curr->y == y){
      //Then remove it!
      done = TRUE;
      if (prev==NULL) wavefront[loc] = curr->next;
      else prev->next = curr->next;
      curr->next = NULL;
    }
  }
}

void Cutter::saveBoundary(int x, int y){
  int    i;
  FILE   *fpt;

  if (!started) return;
  if (plist.num_points == 0) return;
  //Add a seed point here (x, y)
  updatePointList(x, y);
  addSeedPoint();
  initWaveFront(x, y);
  while( updateWaveFront() );
  //Update the point list from the original seed point to this point.
  updatePointList(plist.x[0], plist.y[0]);
  //Save out the boundary minus the duplicated original point.

  printf("saving \"bound.txt\"\n");
  fpt = fopen( "bound.txt", "wb" );
  if ( !fpt ){
    printf("Error: Could not create file: bound.txt\n");
    return;
  }

  plist.num_points--;//Drop the last duplicate point
  fprintf( fpt, "%d\n", plist.num_points);
  for (i=0; i<plist.num_points; i++){
    fprintf( fpt, "%d %d\n", plist.x[i], plist.y[i]);
  }
  fclose( fpt );

  plist.num_points ++;
  removeSeedPoint();
}

void Cutter::addToWaveFront(int x, int y){
  int total_cost;
  int loc;
  loc = x + y*width;

//  printf("DBG002 addToWaveFront()\n");
  total_cost = graph[loc].total_cost;

  //Add the new wave to the wave front
  graph[loc].next = wavefront[ total_cost & WAVE_MOD ];
  wavefront[total_cost & WAVE_MOD] = &graph[loc];
}

//Use the graph to add points from the last seed point to this point.
void Cutter::updatePointList(int px, int py){
  int parent, i, j;
  int curr_x, curr_y;
  int seed_x, seed_y, seed_id;
  //This is a list that will be flipped later
  int x[MAX_NUM_POINTS];
  int y[MAX_NUM_POINTS];

  //Need to make sure the wave front has been expanded enough to handle this
  //point
  if (!started) return;
  if (plist.num_seeds < 1) return;
  if (graph[px + py*width].expanded != TRUE) return;

  //The last seed in the list is the seed whose wavefront is on the graph.
  seed_id = plist.seed[plist.num_seeds-1];
  seed_x = plist.x[seed_id];
  seed_y = plist.y[seed_id];
  i=0;
  curr_x = px;
  curr_y = py;
  while((curr_x != seed_x || curr_y != seed_y) && i<(MAX_NUM_POINTS-seed_id-2)){
    //Add point to the list
    x[i] = curr_x;
    y[i] = curr_y;
    i++;
    parent = graph[curr_x + curr_y*width].parent;
//    printf("curr (%d, %d) seed (%d, %d) p:%d i:%d\n", curr_x, curr_y, seed_x, seed_y, parent, i);
    if (parent == 9){
      printf("Error in Cutter::updatePointList(). seed found at (%d, %d) instead of (%d, %d)\n", curr_x, curr_y, seed_x, seed_y);
      curr_x = seed_x;
      curr_y = seed_y;
      //This is the seed point, end here.
    }
    else{
      getNeighbor8(curr_x, curr_y, &curr_x, &curr_y, parent);
    }
  }
  if (i>= MAX_NUM_POINTS-seed_id-2){
    printf("Error in Cutter::updatePointList(). Not enough memory for pointlist\n");
  }

  for (j=seed_id+1, i--; i>=0; i--, j++){
    plist.x[j] = x[i];
    plist.y[j] = y[i];
  }
  plist.num_points = j;
//  printf("DBG006 MAX: %d  num_points: %d\n", MAX_NUM_POINTS, j);
}

//Advance the wavefront forward
int Cutter::updateWaveFront(){
  int i, j;
  GraphNode *wave;
  int wave_x, wave_y, curr_x, curr_y;
  int wave_loc, curr_loc;
  int cost;

  if (wave_done) return FALSE;
  if (!started) return TRUE;


  //First find the lowest cost node on the wavefront
  //Advance the best_node pointer
  for (j=0; j<WAVE_SIZE && wavefront[best_node] == NULL; j++){
    best_node ++;
    best_node &= WAVE_MOD;
  }
  if (j == WAVE_SIZE){
    wave_done = TRUE;
    return FALSE;
  }

  //Remove that node from the wavefront
  wave = wavefront[best_node];
  wavefront[best_node] = wave->next;
  wave->next = NULL;
  wave_x = wave->x;
  wave_y = wave->y;
  wave_loc = wave_x + wave_y*width;

  //Now expand that node to the next level out
  graph[wave_loc].expanded = TRUE;

  //Look at all neighbors and Set and queue the best one not expanded
  //Now put all the neighbors onto the wavefront.
  for (i=0; i<8; i++){
    getNeighbor8(wave_x, wave_y, &curr_x, &curr_y, i);
    if (curr_x >= width||curr_x < 0 || curr_y >= height||curr_y < 0) continue;
    curr_loc = curr_x  + curr_y*width;
    if (graph[curr_loc].expanded == TRUE) continue;

    cost = graph[wave_loc].total_cost + getCost(wave_x, wave_y, curr_x, curr_y);
    if ( findInWaveFront(curr_x, curr_y) )
     if ( cost < graph[curr_loc].total_cost )
       removeFromWaveFront(curr_x, curr_y);

    if ( !findInWaveFront(curr_x, curr_y) ){
      graph[curr_loc].total_cost = cost;
      graph[curr_loc].parent = (i+4)%8;
      addToWaveFront(curr_x, curr_y);
    }
  }
  return TRUE;

}

void Cutter::initCostGraph(Image *img){
  int i=0, j;
  int val1, val2, val3;
  double max;
  double cost;
  unsigned char *data;
  int p1, p2, p3;
  int *lap;
  Image lap2;

  data = img->data;
  if (height != img->height){
    printf("ERROR: Cutter::initCostGraph()\n");
    return;
  }
  if (width != img->width){
    printf("ERROR: Cutter::initCostGraph()\n");
    return;
  }

  max = 0;
  for (j=0; j<height; j++){
    if (j>0 && j < height-1){
      for (i=0; i<width; i++){
        if (i>0 && i<width-1){
          //Calculate the gradient magnitude using the sobel filter
          val1 =   - (int)data[i-1 + (j-1)*width] 
                   - (int)data[i-1 + j*width]*2 
                   - (int)data[i-1 + (j+1)*width] + 

                   (int)data[i+1 + (j-1)*width] + 
                   (int)data[i+1 + j*width]*2 + 
                   (int)data[i+1 + (j+1)*width] ;

          val2 =   - (int)data[i-1 + (j-1)*width]
                   - (int)data[i +   (j-1)*width]*2
                   - (int)data[i+1 + (j-1)*width] + 

                   (int)data[i-1 + (j+1)*width] + 
                   (int)data[i +   (j+1)*width]*2 + 
                   (int)data[i+1 + (j+1)*width] ;


          cost = sqrt(val1*val1 + val2*val2);


          if (max < cost) max = cost;

          graph[i + j*width].cost = (int)cost;

        }
        else{ graph[i + j*width].cost = 0;}
      }
    }
    else{ graph[i + j*width].cost = 0;}
  }

  max = 128.0/max;
  for (j=0; j<height; j++){
    for (i=0; i<width; i++){
      //Invert the cost graph
      graph[i + j*width].cost = 128 - (int)((double)graph[i + j*width].cost * max);
      if (graph[i + j*width].cost < 0){
        printf("Error %d less than zero\n", graph[i + j*width].cost);
      }
    }
  }


  //Now to calculate the laplacian stuff.
          //Now calculate the laplacian
  lap = new int[width*height];
  for (i=0; i<width*height; i++){ lap[i] = 0; }
  
  for (j=1; j<height-1; j++){
    for (i=1; i<width-1; i++){
      lap[i + j*width] =
                  (int)data[i + j*width]*4
                 -(int)data[i +  (j-1)*width]
                 -(int)data[i +  (j+1)*width]
                 -(int)data[i-1 + j*width]
                 -(int)data[i+1 + j*width];
    }
  }


  lap2.copy(img);
  lap2.clear(127);
  //Now find the zero crossings.
  for (j=0; j<height-1; j++){
    for (i=0; i<width-1; i++){
      p1 = lap[i +j*width];
      p2 = lap[i+1 +j*width];
      p3 = lap[i +(j+1)*width];
      if (p1 == 0){
        lap2.data[i+j*width] = 0;//zero additional cost, because of laplacian
      }
      else if (p1 < 0){
        if (p2>0){
          //zero crossing found
          if (abs(p1) < abs(p2)){ lap2.data[i+j*width] = 0; }
          else{ lap2.data[i+1+j*width] = 0; }
        }
        if (p3>0){
          //zero crossing found
          if (abs(p1) < abs(p3)){ lap2.data[i+j*width] = 0; }
          else{ lap2.data[i+(j+1)*width] = 0; }
        }
      }
      else {
        if (p2<0){
          //zero crossing found
          if (abs(p1) < abs(p2)){ lap2.data[i+j*width] = 0; }
          else{ lap2.data[i+1+j*width] = 0; }
        }
        if (p3<0){
          //zero crossing found
          if (abs(p1) < abs(p3)){ lap2.data[i+j*width] = 0; }
          else{ lap2.data[i+(j+1)*width] = 0; }
        }
        //p1 > 0
      }
    }
  }

  lap2.write("out_lap2.pgm");
  data = lap2.data;
  for (j=0; j<height; j++){
    for (i=0; i<width; i++){
      //Invert the cost graph
      graph[i + j*width].cost += (int)data[i +j*width];//128 - (int)((double)graph[i + j*width].cost * max);
    }
  }

  delete [] lap;

          //Add in the laplacian cost
          //cost += abs(val3);

}

void Cutter::initWaveFront(int seed_x, int seed_y){
  int loc;

  loc = seed_x + seed_y*width;
  deleteWaveFront();
  clearGraph();
  wave_done = FALSE;
  started = TRUE;

  //Add the first node the the wavefront
  wavefront[0] = &graph[loc];
  graph[loc].total_cost = 0;
  best_node = 0;
  graph[loc].parent = 9;

  updateWaveFront();
}

void Cutter::clearGraph(){
  int i;

  for(i=0; i<width*height; i++){
    graph[i].expanded = 0;
    graph[i].total_cost = 0;
    graph[i].parent = 8;
    graph[i].next = NULL;
  }
}

void Cutter::deleteWaveFront(){
  int i;

  for (i=0; i<WAVE_SIZE; i++){
    wavefront[i] = NULL;
  }
  best_node = -1;
  wave_done = FALSE;
}

void Cutter::dumpCostGraph(){
  int i, j;
  Image *im;

  im = new Image(width, height, 5);
  for (j=0; j<height; j++){
    for (i=0; i<width; i++){
      im->data[i + j*width] = graph[i + j*width].cost;
    }
  }

  im->write("out_cost_graph.pgm");
  delete im;
}

void Cutter::dumpWaveFront(){
  int i, j;
  Image *im;
  double max_cost;

  max_cost = 0;
  for (j=0; j<height; j++){
    for (i=0; i<width; i++){
      if (graph[i + j*width].total_cost > max_cost)
        max_cost = graph[i + j*width].total_cost;
    }
  }

  max_cost = 255.0/max_cost;
  im = new Image(width, height, 5);
  for (j=0; j<height; j++){
    for (i=0; i<width; i++){
      im->data[i + j*width] = (unsigned char)(
                                (double)graph[i+j*width].total_cost*max_cost
                              );
    }
  }

  im->write("out_wavefront_image.pgm");
  delete im;
}

void Cutter::dumpPaths(){
  int j, i, parent;
  int seed_x, seed_y;
  int curr_x, curr_y, seed_id;
  GraphNode *scan_wave;

  seed_id = plist.seed[plist.num_seeds-1];
  seed_x = plist.x[ seed_id ];
  seed_y = plist.y[ seed_id ];
  for (j=0; j<WAVE_SIZE; j++){
    for (scan_wave = wavefront[j];
         NULL != scan_wave;
         scan_wave = scan_wave->next){
      curr_x = scan_wave->x;
      curr_y = scan_wave->y;
      i=0;
      while(curr_x != seed_x && curr_y != seed_y && i<(MAX_NUM_POINTS-seed_id-2)){
        i++;
        parent = graph[curr_x + curr_y*width].parent;
        if (parent == 9){
          printf("Error in dumpPaths()\n");
          curr_x = seed_x;
          curr_y = seed_y;
          //This is the seed point, end here.
        }
        else{
          getNeighbor8(curr_x, curr_y, &curr_x, &curr_y, parent);
        }
      }
      if ( i==(MAX_NUM_POINTS-seed_id-2) ){
         printf("\nBAD PATH START\n");
        curr_x = scan_wave->x;
        curr_y = scan_wave->y;
        i=0;
        while(curr_x != seed_x && curr_y != seed_y && i<(MAX_NUM_POINTS-seed_id-2)){
          printf("(%d, %d)  ", curr_x, curr_y );
          i++;
          parent = graph[curr_x + curr_y*width].parent;
          if (parent == 9){
            printf("Error in dumpPaths()\n");
            curr_x = seed_x;
            curr_y = seed_y;
            //This is the seed point, end here.
          }
          else{
            getNeighbor8(curr_x, curr_y, &curr_x, &curr_y, parent);
          }
        }
        printf("\nBAD PATH END\n");
      }
    }
  }
}

int Cutter::findInWaveFront(int x, int y){
  int loc;
  GraphNode *scan_wave;

  loc = graph[x + y*width].total_cost & WAVE_MOD;
  for (scan_wave = wavefront[loc];
       NULL != scan_wave;
       scan_wave = scan_wave->next){
    if (scan_wave->x == x && scan_wave->y == y){
      return TRUE;
    }
  }
  return FALSE;
}


//Cost from going from x1, y1, to x2, y2.
//Generally just the value at x2, y2.
int Cutter::getCost(int x1, int y1, int x2, int y2){
  return graph[x2 + y2*width].cost;
}











//IMAGE
//IMAGE
//IMAGE
//IMAGE
Image::Image(){
  data = NULL;
  height = width = max = type = 0;
}

Image::Image(Image *from_img){
  data = NULL;
  height = width = max = type = 0;

  copy (from_img);
}

Image::Image(int w, int h, int t){
  type = t;
  height = h;
  width = w;
  max = 255;
  if (2 == t || 5 == t)
    data = new unsigned char [w*h];
  if (3 == t || 6 == t)
    data = new unsigned char [w*h*3];
}

/*
 * Creates an image from this signal
 */
Image::Image(double *signal, int size){
  double max_val, min_val, range;
  int hist[MAX_SIGNAL_SIZE];
  int i, x, y, inc; if (size > 1024) size = 1024;
  type = 5;
  height = HIST_DIM;
  width = size;
  max = 255;
  data = new unsigned char [height*width];

  //Find the maximum value for the signal
  max_val = min_val = signal[0];
  for (i=0; i<size; i++){
    if (max_val < signal[i])
      max_val = signal[i];
    if (min_val > signal[i])
      min_val = signal[i];
  }
  if (max_val == min_val){
    max_val = 1.0;
    min_val = -1.0;
  }
    range = max_val - min_val;

  for (i=0; i<size; i++ ){
    hist[i] = (int)(( (signal[i]-min_val) /range)*255.0);
  }

    //color this white
  for (x=0; x<size; x++ ){
    for (y=0; y<HIST_DIM; y++ ){
      data[x + (HIST_DIM-y-1)*size] = 255;
    }
    }

    //Draw zero line
    y = (int) -((min_val / range)*255.0);
    if (y>=0){
        for (x=0; x<size; x++){
            data[x + (HIST_DIM-y-1)*size] = 127;
        }
    }

  for (x=1; x<size; x++){
        if (hist[x-1] > hist[x]) inc = -1;
        else inc = 1;
    for (y=hist[x-1]; y!=hist[x]; y+=inc ){
      //color this black
      data[x + (HIST_DIM-y-1)*size] = 0;
    }
    data[x + (HIST_DIM-hist[x]-1)*size] = 0;
  }
    
}

Image::~Image(){
  if (NULL != data) delete [] data;
}

/*
 * Purpose: Go through and color each connected (r,g,b) region in the image
 *          a different color.
 *
 *          returns the number of shapes colored.
 *
 *          The Id of the shape is equvalent to the Red component value.
 */
unsigned char Image::colorShapes(unsigned char tol, int min_area){
  int x, y, ind;
  unsigned char r, g, b, c_r, c_g, c_b;
  unsigned char br, bg, bb;
  unsigned char i=1;
  int area;

  makeColor();
  if (type != 6 && type != 3){
    printf("Error: Image::colorShapes(). must have color image to color shapes\n");
    return 0;
  } 

  getShapeColor(&r, &g, &b, &br, &bg, &bb, 0);
//  printf("shape color %d, %d, %d \n", (int)r, (int)g, (int)b);//DBG
//  printf("bg color %d, %d, %d min_area: %d\n", (int)br, (int)bg, (int)bb, min_area);//DBG

  getColorFromID(i, &c_r, &c_g, &c_b);
  for(y=0; y<height; y++){
    for(x=0; x<width; x++){
      ind = (x+y*width)*3;
      if ( abs( (int)r-(int)data[ind] )   <= (int)tol &&
           abs( (int)g-(int)data[ind+1] ) <= (int)tol &&
           abs( (int)b-(int)data[ind+2] ) <= (int)tol){
        area = 0;
        seedFill(x, y, r, g, b, c_r, c_g, c_b, tol, &area);
//        printf("area: %d\n", area);
        if (area < min_area){
//          printf("Area too small for shape\n");
          //Area too small for shape
          area = 0;
          seedFill(x, y, c_r, c_g, c_b, br, bg, bb, tol, &area);
        }
        else{
          i++;
          getColorFromID(i, &c_r, &c_g, &c_b);
        }
      }
    }
  }
  return i-1;

  
}


/*
 * Purpose: print out and create image of info
 */
void Image::createInfoImage(int num_shapes){
  int i;
  int cx, cy;
  int area;
  double orient;

  for (i=1; i<=num_shapes; i++){
    area = markCenterOfGravityColor(&cx, &cy, (unsigned char) i);
    orient = markOrientation(i, cx, cy);
    printf("Shape %d| area: %d, CoG: (%d, %d), Oriet: %f\n", i, area, cx, cy, orient);
  }
  //getArea
  //centerofgravity
  //orientation
}

void Image::clear(unsigned char val){
  int ind;

  if (2 == type || 5 == type){
    for (int y=0; y<height; y++ ){
      for (int x=0; x<width; x++){
        ind = y*width +x;
        data[ind] =  val;
      }
    }
  }

  else if (3 == type || 6 == type){
    for (int y=0; y<height; y++ ){
      for (int x=0; x<width; x++){
        ind = (y*width +x)*3;
        data[ind  ] =  val;
        data[ind+1] =  val;
        data[ind+2] =  val;
      }
    }
  }

}


void Image::copy(Image *from_img){
  int dim, loc;

  if (NULL != data) delete [] data;

  height = from_img->height;
  width  = from_img->width;
  type   = from_img->type;
  max    = from_img->max;
  if (3 == type || 6 == type) dim = width*height*3;
  else dim = width*height*1;
  //allocate mem, and copy
  data = new unsigned char [dim];
  for (loc=0; loc<dim; loc++){
    data[loc] = from_img->data[loc];
  }

}

/*
 * Purpose: Get the difference of the two images and assign it to this one.
 */
void Image::diff(Image *img){
  int x, y;
  if (height != img->height || width != img->width){
    printf("Error in Image::diff(Image *img). Images must have the same dimensions\n");
    return;
  }

  img->makeGray();
  makeGray();

  for(y=0; y<height; y++){
    for(x=0; x<width; x++){
      data[x + y*width] = (unsigned char) abs( (int)data[x + y*width] - (int)img->data[x + y*width]);
    }
  }
}

void Image::dilate(int size){
  int i, j, l, m;
  int offset;
  unsigned char *new_data;

  makeGray();
  offset = (size-1)/2;
  new_data = new unsigned char [height*width];
  for(i=0; i<height*width; i++){
    new_data[i] = 0;
  }

  for(i=offset; i<width-offset; i++){
    for(j=offset; j<height-offset; j++){

      //If the pixel is set.
      if(data[i + j*width] > 0){
        for(l=0; l<size; l++){
          for(m=0; m<size; m++){
//            if(filter[l + m*size] > tol)
//              new_data[i-offset+l+(j-offset+m)*width] = filter[l + m*size];
              new_data[i-offset+l+(j-offset+m)*width] = 255;//filter[l + m*size];
          }
        }
      }
    }
  }
  delete [] data;
  data = new_data;
}

void Image::erode(int size){
  int i, j, l, m;
  int offset;
  int set;
  unsigned char *new_data;

  makeGray();
  offset = (size-1)/2;
  new_data = new unsigned char [height*width];
  for(i=0; i<height*width; i++){
    new_data[i] = 0;
  }

  for(i=offset; i<width-offset; i++){
    for(j=offset; j<height-offset; j++){

      set = 1;
      for(l=0; l<size; l++){
        for(m=0; m<size; m++){
          if( data[i-offset+l+(j-offset+m)*width] == 0){
              set = 0;
              m = l = size;//break out
          }
        }
      }
      //Now set the pixel in the outgoing image if needs be.
      if (1 == set)
        new_data[i + j*width] = data[i + j*width];
      else
        new_data[i + j*width] = 0;
    }
  }
  delete data;
  data = new_data;
}

/*
 * Purpose: Makes the image a residual of a predicted encoding scheme.
 * -predictor
 *   a b c
 *   d e
 *       e = (a+b+c+d)/4;
 *  if flag == 1 => encode;
 *  if flag == 0 => decode;
 */
void Image::predCode1(int flag){
  int i, j;
  unsigned char *new_data;
  int pred, num_pred;

  if (flag == 1){
    new_data = new unsigned char[width*height];
  }
  else{
    new_data = data;
  }
  makeGray();

  for (j=0; j<height; j++){
    for (i=0; i<width; i++){
      pred = 0;
      num_pred = 0;
      if (i>0){
        pred += (int)data[i-1 + j*width];//d
        num_pred ++;
      }
      if (j>0){
        pred += (int)data[i + (j-1)*width];//b
        num_pred ++;
        if (i>0){
          pred += (int)data[i-1 + (j-1)*width];//a
          num_pred ++;
        }
        if (i<width-2){
          pred += (int)data[i+1 + (j-1)*width];//c
          num_pred ++;
        }
      }

      if (num_pred == 0){
        new_data[i + j*width] =data[i + j*width];
        continue;
      }

      pred = pred/num_pred;

      if (flag == 1){//encode
        if (data[i + j*width] < (unsigned char)pred)
          new_data[i + j*width] =(unsigned char)( 256 + (int)data[i + j*width] - pred);
        else
          new_data[i + j*width] = data[i + j*width] - (unsigned char)pred;
      }
      else{ //decode
        if( 256-(unsigned char)pred < data[i + j*width])
          new_data[i + j*width] = (unsigned char)((int)data[i + j*width] + pred - 256);
        else
          new_data[i + j*width] = data[i + j*width] + (unsigned char)pred;
      }
    }
  }
  if (flag == 1) delete data;
  data = new_data;
}

/*
 * Purpose: Makes the image a residual of a predicted encoding scheme.
 * -predictor
 *   a b c
 *   d e
 *       e = (a+b+c+d)/4;
 *  if flag == 1 => encode;
 *  if flag == 0 => decode;
 */
void Image::predCode2(int flag){
  int i, j;
  unsigned char *new_data;
  int pred, num_pred;

  if (flag == 1){
    new_data = new unsigned char[width*height];
  }
  else{
    new_data = data;
  }
  makeGray();

  for (j=0; j<height; j++){
    for (i=0; i<width; i++){
      pred = 0;
      num_pred = 0;
      if (i>1){
        //two step predictor
        pred += 2*(int)data[i-1 + j*width] - (int)data[i-2 + j*width];//d
        num_pred ++;
      }
      else if (i>0){
        pred += (int)data[i-1 + j*width];//d
        num_pred ++;
      }

      if (j>1){
        pred += 2*(int)data[i + (j-1)*width] - (int)data[i + (j-2)*width];//b
        num_pred ++;
        if (i>0){
          pred += 2*(int)data[i-1 + (j-1)*width] - (int)data[i-1 + (j-2)*width] ;//a
          num_pred ++;
        }
        if (i<width-2){
          pred += 2*(int)data[i+1 + (j-1)*width]- (int)data[i+1 + (j-2)*width];//c
          num_pred ++;
        }
      }
      else if (j>0){
        pred += (int)data[i + (j-1)*width];//b
        num_pred ++;
        if (i>0){
          pred += (int)data[i-1 + (j-1)*width];//a
          num_pred ++;
        }
        if (i<width-2){
          pred += (int)data[i+1 + (j-1)*width];//c
          num_pred ++;
        }
      }
      if (num_pred == 0){
        new_data[i + j*width] =data[i + j*width];
        continue;
      }

      if (pred < 0) pred = 0;

      pred = pred/num_pred;

      if (flag == 1){//encode
        if (data[i + j*width] < (unsigned char)pred)
          new_data[i + j*width] =(unsigned char)( 256 + (int)data[i + j*width] - pred);
        else
          new_data[i + j*width] = data[i + j*width] - (unsigned char)pred;
      }
      else{ //decode
        if( 256-(unsigned char)pred < data[i + j*width])
          new_data[i + j*width] = (unsigned char)((int)data[i + j*width] + pred - 256);
        else
          new_data[i + j*width] = data[i + j*width] + (unsigned char)pred;
      }
    }
  }
  if (flag == 1) delete data;
  data = new_data;
}

/*
 * Purpose: Equalizes a gray image
 */
void Image::equalize(){
  int i, j, x, y;
  double max;
  double lup[HIST_DIM];
  double *pdf = new double[HIST_DIM];

  //First check the image type
  if (type == 6 || type == 3){
    printf("Error: Image::equalize(). Image must be gray scale.\n");
    return;
  } 

  //Second create the PDF (probability density function) for this image.
  getPDF(pdf, HIST_DIM, &max);

  //Third create a look up table for the color intensities transformation.
  for (i=0; i<HIST_DIM; i++){
    lup[i] = 0.0;
    for (j=0; j<=i; j++){
      lup[i] += pdf[j];
    }
    lup[i] *= (HIST_DIM-1);
    //printf("lup[%d]: %f\n",i ,  lup[i]);
  }

  //Fourth now use the lookup table to get the new image.
  for (y=0; y<height; y++ ){
    for (x=0; x<width; x++){
      data[x + y*width] = (unsigned char) lup[data[x + y*width]];
    }
  }

  if (NULL != pdf) delete [] pdf;
}

/*
 * Purpose: flips the image vertically
 */
void Image::flip(){
  unsigned char r;
  int i, j, h_min_j, ind1, ind2;
  
  makeGray();
  for (j=0; j<height/2; j++){
    h_min_j = height - 1 - j;
    for (i=0; i<width; i++){
      ind1 = i + j*width;
      ind2 = i + h_min_j*width;
      r = data[ind1];
      data[ind1] = data[ind2];
      data[ind2] = r;
    }
  }

  /*
  unsigned char r, g, b;
  int i, j, h_min_j, ind1, ind2;

  makeColor();
  for (j=0; j<height/2; j++){
    h_min_j = height - 1 - j;
    for (i=0; i<width; i++){
      ind1 = (i + j*width)*3;
      ind2 = (i + h_min_j*width)*3;
      r = data[ind1];
      g = data[ind1+1];
      b = data[ind1+2];
      data[ind1] = data[ind2];
      data[ind1+1] = data[ind2+1];
      data[ind1+2] = data[ind2+2];
      data[ind2] = r;
      data[ind2+1] = g;
      data[ind2+2] = b;
    }
  }
  */

}

double Image::markOrientation(int i, int cx, int cy){
  double angle;
  double u11, u02, u20;
  int x, y;
  int total_x, total_y, count;
  makeColor();
  double vx, vy;

  total_x = total_y = count = 0;
  u11 = u20 = u02 = 0.0;
  for(y=0; y<height; y++){
    for(x=0; x<width; x++){
      if ( (data[(x + y*width)*3 + 2 ] - i)  == 0){
        u11 += (double)(x - cx)*(y - cy);
        u20 += (double)(x - cx)*(x - cx);
        u02 += (double)(y - cy)*(y - cy);
      }
    }
  }
  //angle = 0.5*atan(2.0*u11/(u20-u02));
  angle = 0.5*atan2(2.0*u11,(u20-u02));

  vx = 30*cos(angle);
  vy = 30*sin(angle);

  drawLine(cx-(int)vx, cy-(int)vy, cx+(int)vx, cy+(int)vy, 0, 128, 255);

  return angle;
}

/*
 * Purpose: Finds the center of gravity for all pixels whose value is greater
 *          than tol.
 */
int Image::markCenterOfGravityColor(int *cx, int *cy, unsigned char i){
  int x, y;
  int total_x, total_y, count;
  makeColor();

  total_x = total_y = count = 0;
  for(y=0; y<height; y++){
    for(x=0; x<width; x++){
      if ( (data[(x + y*width)*3 + 2 ] - i)  == 0){
        total_x += x;
        total_y += y;
        count ++;
      }
    }
  }
  *cx = (int)((double)total_x/(double)count);
  *cy = (int)((double)total_y/(double)count);

  drawLine(*cx-6, *cy, *cx+6, *cy, 255, 255, 255);
  drawLine(*cx, *cy-6, *cx, *cy+6, 255, 255, 255);

  return count;
}


/*
 * Purpose: draws a pixel line on the image from x0, y0, to x1, y1
 *          using the midpoint line algorithm.
 */
void Image::drawLine(int x0, int y0, int x1, int y1, unsigned char r, unsigned char g, unsigned char b) {
  int dy = y1 - y0;
  int dx = x1 - x0;
  int stepx, stepy;
  int fraction;

  if (dy < 0) { dy = -dy;  stepy = -1; } else { stepy = 1; }
  if (dx < 0) { dx = -dx;  stepx = -1; } else { stepx = 1; }
  dy <<= 1;                     // dy is now 2*dy
  dx <<= 1;                     // dx is now 2*dx

  if (x0 < width && x0 > 0 && y0 < height && y0 > 0){
    data[(x0+ y0*width)*3    ] = r;
    data[(x0+ y0*width)*3 + 1] = g;
    data[(x0+ y0*width)*3 + 2] = b;
  }
  if (dx > dy) {
      fraction = dy - (dx >> 1);          // same as 2*dy - dx
      while (x0 != x1) {
          if (fraction >= 0) {
              y0 += stepy;
              fraction -= dx;                // same as fraction -= 2*dx
          }
          x0 += stepx;
          fraction += dy;                    // same as fraction -= 2*dy
          if (x0 < width && x0 > 0 && y0 < height && y0 > 0){
            data[(x0+ y0*width)*3    ] = r;
            data[(x0+ y0*width)*3 + 1] = g;
            data[(x0+ y0*width)*3 + 2] = b;
          }
      }
  } else {
      fraction = dx - (dy >> 1);
      while (y0 != y1) {
          if (fraction >= 0) {
              x0 += stepx;
              fraction -= dy;
          }
          y0 += stepy;
          fraction += dx;
          if (x0 < width && x0 > 0 && y0 < height && y0 > 0){
            data[(x0+ y0*width)*3    ] = r;
            data[(x0+ y0*width)*3 + 1] = g;
            data[(x0+ y0*width)*3 + 2] = b;
          }
      }
  }


}

/*
 * Purpose: Finds the center of gravity for all pixels whose value is greater
 *          than tol.
 */
void Image::getCenterOfGravity(int *cx, int *cy, unsigned char tol){
  int x, y;
  int total_x, total_y, count;
  makeGray();

  total_x = total_y = count = 0;
  for(y=0; y<height; y++){
    for(x=0; x<width; x++){
      if (data[x + y*width] > tol){
        total_x += x;
        total_y += y;
        count ++;
      }
    }
  }
  *cx = (int)((double)total_x/(double)count);
  *cy = (int)((double)total_y/(double)count);
}

/*
 * Purpose: returns a pdf (probability density function) for the historgram of
 *          this image scaled to,  area = 1.0;
 */
void Image::getPDF(double *pdf, int n, double *max){
  double val, total;
  int x, y, i;

  if (2 != type && 5 != type){
    printf("Error: Image::getPDF(). must have gray image to get gray PDF\n");
    return;
  }
  if (n != HIST_DIM){
    printf("Error: Image::getPDF(). Cannot handle a pdf of size other than %d\n", HIST_DIM);
    return;
  }

  for (i=0; i<n; i++){
    pdf[i] = 0.0;
  }

  *max = 0.0;
  total = height*width;
  for (y=0; y<height; y++ ){
    for (x=0; x<width; x++){
      val = ++pdf[data[x + y*width]];
      if (val > *max) *max = val;
    }
  }

  //Normalize it afterwards for better floating point calculation
  //rather than adding little increments each step.
  for (i=0; i<n; i++){
    pdf[i] /= total;
  }

}

void Image::getItemBounds(unsigned char id, int *bot_x, int *bot_y,
                          int *top_x, int *top_y){
  int x, y, ind;

  makeColor();
  *bot_x = width;
  *bot_y = height;
  *top_x = 0;
  *top_y = 0;
  for(y=0; y<height; y++){
    for(x=0; x<width; x++){
      ind = (x+y*width)*3;
      if ( data[ind+2] == id){
        if (x > *top_x) *top_x = x;
        if (x < *bot_x) *bot_x = x;
        if (y > *top_y) *top_y = y;
        if (y < *bot_y) *bot_y = y;
      }
    }
  }
}

unsigned char Image::getBoundAverage(int bot_x, int bot_y,
                                    int top_x, int top_y){
  int x, y, ind, total, count;

  count = 0;
  total = 0;
  for(y=bot_y; y<top_y; y++){
    for(x=bot_x; x<top_x; x++){
      ind = x+y*width;
      total += (int)data[ind];
      count ++;
    }
  }
  return (int)((double)total/(double)count);
}

void Image::unionThreshRegion(Image *img, int bot_x, int bot_y, int top_x,
                              int top_y, int range, unsigned char t){
  int x, y, ind, l=0, m=0;
  double count, ave;

  makeGray();
  img->makeGray();
  for(y=bot_y+range; y<top_y-range; y++){
    for(x=bot_x+range; x<top_x-range; x++){

      //do averaging filter
      count = 0.0;
      ave = 0.0;//a[i + j*w];
      for(l=x-range; l<x+range; l++){
        for(m=y-range; m<y+range; m++){
          ave += (double)img->data[l + m*width];
          count ++;
        }
      }
      ave /= count;

      ind = l+m*width;

      if (img->data[ind] > t){
        data[ind] = 255;
      }

    }
  }
}

double *Image::getDouble(double offset){
  int i;
  double *d_data = new double[height*width];

  makeGray();
  for (i=0; i<height*width; i++){
    d_data[i] = (double)data[i] + offset;
  }
  return d_data;
}

void Image::invert(){
  int i;
  makeGray();
  for (i=0; i<height*width; i++){
    data[i] = 255 - data[i];
  }
}

/*
 * Purpose: We'll get the color for the shapes and set our own bg color
 */
void Image::getShapeColor(unsigned char *r, unsigned char *g, unsigned char *b,
                       unsigned char *br, unsigned char *bg, unsigned char *bb,
                       unsigned char tol){
  int x, y, ind;

  if (2 == type || 5 == type){
    printf("Error: Image::getShapeColor(). You must first make this a color image.\n");
    return;
  }

  /*
  *br = data[0];
  *bg = data[1];
  *bb = data[2];
  */
  *br = 0;
  *bg = 0;
  *bb = 0;
  

  for (y=0; y<height; y++ ){
    for (x=0; x<width; x++){
      ind = (x + y*width)*3;
      if ( abs( (int)(*br)-(int)data[ind] )   > (int)tol ||
           abs( (int)(*bg)-(int)data[ind+1] ) > (int)tol ||
           abs( (int)(*bb)-(int)data[ind+2] ) > (int)tol){
        *r = data[ind];
        *g = data[ind+1];
        *b = data[ind+2];
        x = width;
        y = height;
      }
    }
  }

  //Now we'll recolor the back ground.
  for (y=0; y<height; y++ ){
    for (x=0; x<width; x++){
      ind = (x + y*width)*3;
      if ( abs( (int)(*br)-(int)data[ind] )   <= (int)tol ||
           abs( (int)(*bg)-(int)data[ind+1] ) <= (int)tol ||
           abs( (int)(*bb)-(int)data[ind+2] ) <= (int)tol){
        data[ind]   = *br;
        data[ind+1] = *bg;
        data[ind+2] = *bb;
      }
    }
  }

}

/*
 * Purpose: pads the image with black around the borders.
 */
void Image::pad(int new_width, int new_height){
  int i, x_off, y_off, nx, ny;
  unsigned char *new_data;

  if (NULL == data) return;

  if (width > new_width || height > new_height){
    printf("Error: size for pad not big enough\n");
    return;
  }

  makeGray();

  new_data = new unsigned char[new_height*new_width];

  for (i=0; i<new_height*new_width; i++){
    new_data[i] = 0;
  }
  x_off = (new_width-width)/2;
  y_off = (new_height-height)/2;

  for (ny = 0; ny<height; ny++){
    for (nx = 0; nx<width; nx++){
      new_data[nx+x_off + (ny+y_off)*new_width] = data[nx + ny*width];
    }
  }
  delete [] data;
  data = new_data;
  height = new_height;
  width = new_width;
  
}

/*
 * Purpose: rotates the image the specified amount.
 * Note:    The value passed in is assumed to be in degrees
 */
void Image::rotate(double theta){
  unsigned char *new_data;
  int nx, ny;
  double x1, x2, y1, y2; 
  double sint, cost;
  double xc, yc, tx, ty;
  double sample1, sample2;
  int ix2, iy2;

  makeGray();

  //convert to radians
  theta *= 6.28/360.0;
  sint = sin(theta);
  cost = cos(theta);

  new_data = new unsigned char [height*width];

  xc = (double)width/2.0;
  yc = (double)height/2.0;

  //To do the backwards mapping, we'll just take samples in the new image
  //and rotate back to the previous to get the value
  for (ny = 0; ny<height; ny++){
    for (nx = 0; nx<width; nx++){

      x1 = (double)nx;
      y1 = (double)ny;
      x2 = (x1-xc)*cost - (y1-yc)*sint + xc;
      y2 = (x1-xc)*sint + (y1-yc)*cost + yc;
      ix2 = (int)x2;
      iy2 = (int)y2;
      tx = x2 - (double)ix2;
      ty = y2 - (double)iy2;
      
      if (ix2+1<width && ix2>=0 && iy2+1<height && iy2>=0){
        sample1 = (1.0-tx)*(double)data[ix2 + iy2*width] +
                        tx*(double)data[ix2+1 + iy2*width];
        sample2 = (1.0-tx)*(double)data[ix2 + (iy2+1)*width] +
                        tx*(double)data[ix2+1 + (iy2+1)*width];
        new_data[nx + ny*width] = (unsigned char)((1-ty)*sample1 + ty*sample2);
      }
      else{
        new_data[nx + ny*width] = 128;
      }

    }
  }
  //Assign the new image
  delete [] data;
  data = new_data;
}

void Image::makeSinusoid(int size, double freq, double mag, int axis){
  int i, j;
  double N;
  double half = mag/2.0;
  unsigned char val;

  if (data != NULL) delete [] data;
  data = new unsigned char[size*size];
  height = width = size;
  type = 5;
  max = (int)mag;

  N = (double) size;
  if (axis == X_AXIS){
    for (i=0; i<size; i++){
      val = (unsigned char)(half*sin( (freq * (double)i)/N) + half);
      for (j=0; j<size; j++){
        data[i + j*size] = val;
      }
    }
  }
  else if (axis == Y_AXIS){
    for (j=0; j<size; j++){
      val = (unsigned char)(half*sin( (freq * (double)j)/N) + half);
      for (i=0; i<size; i++){
        data[i + j*size] = val;
      }
    }
  }

}

/*
 * Assigns the image the magnitued plot of the 2d signal
 */
void Image::makeMag(Complex *D_Data, int size){
  int i, j;
  double mag;
  double d_max, d_min;

  if (data != NULL) delete [] data;
  data = new unsigned char[size*size];
  height = width = size;
  type = 5;
  max = 255;

  //Set max and min
  d_max = d_min = D_Data[0].getMag();
  for (i=1; i<size*size; i++){
    mag = D_Data[i].getMag();
    if (d_max < mag) d_max = mag;
    else if (d_min > mag) d_min = mag;
  }

  for (i=0; i<size; i++){
    for (j=0; j<size; j++){
      data[i + j*size] = (unsigned char)( ((D_Data[i+j*size].getMag() - d_min)/(d_max-d_min))*(double)max);
    }
  }

  swapQuads();
}

/*
 * Assigns the image the magnitued plot of the 2d signal
 */
void Image::makePhase(Complex *D_Data, int size){
  int i, j;
  double mag;
  double d_max, d_min;

  if (data != NULL) delete [] data;
  data = new unsigned char[size*size];
  height = width = size;
  type = 5;
  d_max = 255;

  //Set max and min
  d_max = d_min = D_Data[0].getPhase();
  for (i=1; i<size*size; i++){
    mag = D_Data[i].getPhase();
    if (d_max < mag) d_max = mag;
    else if (d_min > mag) d_min = mag;
  }

  for (i=0; i<size; i++){
    for (j=0; j<size; j++){
      data[i + j*size] = (unsigned char)( ((D_Data[i+j*size].getPhase() - d_min)/(d_max-d_min))*(double)max);
    }
  }

  swapQuads();
}

void Image::mask(Image *m){
  int i, j;
  makeGray();
  m->makeGray();
  for (i=0; i<m->width; i++){
    for (j=0; j<m->height; j++){
      if (m->data[i + j*m->width] == 0){
        if (i<width && j<height){
          data[i + j*width] = 0;
        }
      }
    }
  }
}

void Image::signalToImage2D(double *d_data, int w, int h){
  int i, j;
  double mag;
  double d_max, d_min;

  if (data != NULL) delete [] data;
  data = new unsigned char[w*h];
  height = h;
  width = w;
  type = 5;
  max = 255;

  //Set max and min
  d_max = d_min = d_data[0];
  for (i=1; i<w*h; i++){
    mag = d_data[i];
    if (d_max < mag) d_max = mag;
    else if (d_min > mag) d_min = mag;
  }

  for (i=0; i<w; i++){
    for (j=0; j<h; j++){
      data[i + j*w] = (unsigned char)( ((d_data[i+j*w] - d_min)/(d_max-d_min))*(double)max);
    }
  }
}

void Image::swapQuads(){
  int i, j;
  int halfw, halfh;
  unsigned char *new_data;


  //now copy the image quadrants to where they're supposed to be.
  new_data = new unsigned char[width*height];
  halfw = width/2;
  halfh = height/2;
  for (i=0; i<width; i++){
    for (j=0; j<height; j++){

      if (i<halfw){
        if (j<halfh)
          new_data[i + j*width] = data[i + halfw + (j+halfh)*width];
        else
          new_data[i + j*width] = data[i + halfw + (j-halfh)*width];
      }
      else{
        if (j<halfh)
          new_data[i + j*width] = data[i - halfw + (j+halfh)*width];
        else
          new_data[i + j*width] = data[i - halfw + (j-halfh)*width];
      }

    }
  }

  delete [] data;
  data = new_data;
}

void Image::threshold(unsigned char thresh){
  int i;
  int dim = width*height;

  makeGray();

  for (i=0; i<dim; i++){
    if (data[i] < thresh) data[i] = 0;
    else data[i] = 255;
  }

}

/************************************************************************
writeImage - writes image out to the given filename in binary PPM format

note: forced_n being non negative will cause mica to only save out that
      particular image in the series.

************************************************************************/
int Image::write(char *filename){
  int    i, j;
  FILE   *fpt;
  unsigned char *save_line;
  int count;
  

  printf("saving %s... ", filename);
  fpt = fopen( filename, "wb" );
  if ( !fpt ){
    printf("Error: Could not create file: %s\n", filename);
    return FALSE;
  }

  if (2 == type){//gray scale
    fprintf( fpt, "P2\n" );
    fprintf( fpt, "# Created by the illustrious Mike D. Smith\n" );
    fprintf( fpt, "%d %d\n", width, height);
    fprintf( fpt, "%d\n", max);
    for( j=0, count=0; j<height; j++ ){
      for( i=0; i<width; i++, count++){
        if (count>10){//no line in the file should be longer than 70 chars
          fprintf( fpt, "\n");
          count = 0;
        }
        fprintf( fpt, "%d ", data[i + j*width]);
      }
    }
    fclose( fpt );
    printf("done\n");
    return TRUE;
  }


  else if (3 == type){
    fprintf( fpt, "P3\n" );
    fprintf( fpt, "# Created by the illustrious Mike D. Smith\n" );
    fprintf( fpt, "%d %d\n", width, height);
    fprintf( fpt, "%d\n", max);
    for( j=0, count=0; j<height; j++ ){
      for( i=0; i<width; i++, count++){
        if (count>4){//no line in the file should be longer than 70 chars
          fprintf( fpt, "\n");
          count = 0;
        }
        fprintf( fpt, "%d %d %d ", data[(i + j*width)*3], data[(i + j*width)*3 + 1], data[(i + j*width)*3 + 2]);
      }
    }
    fclose( fpt );
    printf("done\n");
    return TRUE;
  }

  else if (5 == type){
    fprintf( fpt, "P5\n" );
    //if ( HeaderInfo!=NULL ) fprintf( F, "# %s\n", HeaderInfo );
    //else fprintf( F, "# Grayscale Image\n" );
    fprintf( fpt, "%d %d\n%d\n", width, height, max );
    for ( i=0; i<height; i++ ) {
      save_line = &data[i*width];
      if ( fwrite( (void*)save_line, 1, (size_t)width, fpt ) !=
           (unsigned int)width ){
        printf("fail to write\n");
        return FALSE;
      }
    }
    printf("done\n");
    fclose( fpt );
    return TRUE;
  }

  else if (6 == type){
    fprintf( fpt, "P6\n" );
    //if ( HeaderInfo!=NULL ) fprintf( F, "# %s\n", HeaderInfo );
    //else fprintf( F, "# Grayscale Image\n" );
    fprintf( fpt, "%d %d\n%d\n", width, height, max );
    for ( i=0; i<height; i++ ) {
      save_line = &data[i*width*3];
      if ( fwrite( (void*)save_line, 1, (size_t)(width*3), fpt ) !=
           (unsigned int)(width*3)  ){
        puts("fail to write");
        return FALSE;
      }
    }
    printf("done\n");
    fclose( fpt );
    return TRUE;
  }
  else{
    printf("Failure to write PPM type %d not supported\n", type);
    return FALSE;
  }
}

/*
 * Purpose: read - reads a PNM file into an unsigned byte array
 * 
 * filename = the image file you wish to open.
 * width  = the image dimension in the x direction.
 * height  = the image dimension in the y direction.
 */
int Image::read(char *filename) {
  FILE *fp;
  register int w, h;
  int ind;
  char buffer[250];

  fp = fopen(filename, "rb");
  if (fp == NULL) {
    printf("Could not open file %s\n", filename);
    return FALSE;
  }
  printf("Reading %s\n", filename);

    //get the ppm format type
  fscanf(fp,"P%d\n", &type);


  //skip comments and empty spaces.
  buffer[0] = '#';
  while(buffer[0] == '#' || buffer[0] == ' ' || buffer[0] == '\n')
    fgets(buffer,250,fp);

    //then get the width and height
  sscanf(buffer,"%d%d",&w,&h);
//  printf("Image width: %d, height: %d\n", w, h);

  width = w;
  height = h;

    //get the maximum value for the ppm file (should be 255)
  fgets(buffer,250,fp);
  sscanf(buffer,"%d",&max);
//  printf("max pixel value is: %d.\n", max);

  if (type == 2){//Gray scale image
    data = new unsigned char [w * h];
    // code to handle P2 type ppms (ascII type)
    for (int y=0; y<h; y++ ){
      for (int x=0; x<w; x++){
        int c;
        fscanf(fp,"%d",&c);
        data[y*w +x] = (unsigned char)( ((double)c/(double)max)*255.0);
      }
    }
  }

  else if (type == 3){
    data = new unsigned char [w * h * 3];
    // code to handle P3 type ppms (ascII type)
    for (int y=0; y<h; y++ ){
      for (int x=0; x<w; x++){
        int r,g,b;
        fscanf(fp,"%d%d%d",&r,&g,&b);
        ind = (y*w +x) * 3;
        data[ind]     = (unsigned char)( ((double)r/(double)max)*255.0);
        data[ind + 1] = (unsigned char)( ((double)g/(double)max)*255.0);
        data[ind + 2] = (unsigned char)( ((double)b/(double)max)*255.0);
      }
    }
  }
  else if (type == 5){
    data = new unsigned char [w * h];
    fread(data, 1, h * w, fp );
  }
  else if (type == 6){
    data = new unsigned char [w * h * 3];
    fread(data, 1, h * w * 3, fp );
  }

  else{
    printf(" *.ppm File must be of type P2, P3, P5 or P6 to be read.\n");
    return FALSE;
  }

  fclose(fp);
//  printf("Finished Reading %s\n", filename);
  return TRUE;
}

/*
 *  Grey = 0.299 Red + 0.587 Green + 0.114 Blue
 */
void Image::makeGray() {
  int ind;
  unsigned char *gray_data;

  if (3 == type || 6 == type){
    //convert to gray.
    gray_data = new unsigned char [height*width];
    type = 5;

    for (int y=0; y<height; y++ ){
      for (int x=0; x<width; x++){
        ind = (y*width +x) * 3;
        gray_data[x + y*width] =  (unsigned char)( 0.299*(double)data[ind] +
                                                   0.587*(double)data[ind + 1]+
                                                   0.114*(double)data[ind + 2]);
      }
    }
    delete [] data;
    data = gray_data;
  }
}

/*
 * Don't do this.
 *  Red =   Gray/0.299 
 *  Green = Gray/0.587 
 *  Blue =  Gray/.114
 */
void Image::makeColor() {
  int ind;
  unsigned char *color_data;

  if (2 == type || 5 == type){
    //convert to color.
    color_data = new unsigned char [3*width*height];
    type = 6;

    for (int y=0; y<height; y++ ){
      for (int x=0; x<width; x++){
        ind = y*width +x;
        color_data[ind*3] =    data[ind];
        color_data[ind*3+1] =  data[ind];
        color_data[ind*3+2] =  data[ind];
      }
    }
    delete [] data;
    data = color_data;
  }
}

void Image::makePowerOf2(int new_size){
  makeGray();

  new_size = findPowerOf2(new_size);
  if (new_size == width && width == height) return;
  pad(new_size, new_size);
}

/*
 */
Image* Image::getEqualizeTableImage () {
  Image *table_img;
  double max;
  double *pdf = new double[HIST_DIM];
  double lup[HIST_DIM];
  int i, j, x, y;

  getPDF(pdf, HIST_DIM, &max);

  for (i=0; i<HIST_DIM; i++){
    lup[i] = 0.0;
    for (j=0; j<=i; j++){
      lup[i] += pdf[j];
    }
    lup[i] *= (HIST_DIM-1);
  }



  table_img = new Image(HIST_DIM, HIST_DIM, 5);
  for (x=0; x<HIST_DIM; x++){
    y=0;
    for (; y<(int)lup[x]; y++ ){
      //color this black
      table_img->data[x + (HIST_DIM-y-1)*HIST_DIM] = 127;
    }
    for (; y<HIST_DIM; y++ ){
      //color this gray
      table_img->data[x + (HIST_DIM-y-1)*HIST_DIM] = 255;
    }
  }

  delete [] pdf;
  return table_img;
}

/*
 *  Gray = 0.299 Red + 0.587 Green + 0.114 Blue
 */
Image* Image::getHistImage (int flag) {
  int x, y, i;
  int hist[HIST_DIM];
  Image *hist_img;
  int offset=0;
  int val, max=0;

  for (int i=0; i<HIST_DIM; i++ ){
    hist[i] = 0;
  }

  switch(flag){
    case GRY://gray
      if (2 != type && 5 != type){
        printf("Error: Image::getHistImage(). must have gray image to get gray histogram\n");
        return NULL;
      }
      for (y=0; y<height; y++ ){
        for (x=0; x<width; x++){
          val = ++hist[data[x + y*width]];
          if (val > max) max = val;
        }
      }
      break;

    case BLU://blue
      offset++;
    case GRN://green
      offset++;
    case RED://red
      if (6 != type && 3 != type){
        printf("Error: Image::getHistImage(). must have color image to get gray histogram\n");
        return NULL;
      }
      for (y=0; y<height; y++ ){
        for (x=0; x<width; x++){
          val = ++hist[data[(x+y*width)*3 + offset]];
          if (val > max) max = val;
        }
      }
      break;
    default:
        printf("Error: Image::getHistImage(). Histogram type not recognized\n");
        return NULL;
      break;
  }

  for (i=0; i<HIST_DIM; i++ ){
    hist[i] = (int) (((double)hist[i]/(double)(max+20))*255.0);
  }

  hist_img = new Image(HIST_DIM, HIST_DIM, 5);
  for (x=0; x<HIST_DIM; x++){
    y=0;
    for (; y<hist[x]; y++ ){
      //color this black
      hist_img->data[x + (HIST_DIM-y-1)*HIST_DIM] = 0;
    }
    for (; y<HIST_DIM; y++ ){
      //color this white
      hist_img->data[x + (HIST_DIM-y-1)*HIST_DIM] = 255;
    }
  }


  return hist_img;
}

/*
 * Purpose: Fills neighboring regions of the same color(r, g, b) within tol
 *          with (f_r, f_g, f_b);
 */
void Image::seedFill(int x, int y,
                     unsigned char r, unsigned char g, unsigned char b,
                     unsigned char f_r, unsigned char f_g, unsigned char f_b,
                     unsigned char tol, int *area){
  int ind;

  if (type != 6 && type != 3){
    printf("Error: Image::seedFill(). must have color image to color shapes\n");
    return;
  }

  //Check edges of image
  if (x < 0 || x >= width || y < 0 || y >= height)
    return;

  ind = (x+y*width)*3;

  //Already this color
  if ( f_r == data[ind] && 
       f_g == data[ind+1] && 
       f_b == data[ind+2])
    return;

  //Eccedes tolerance for color that we're filling
  if ( abs( (int)r-(int)data[ind] )   > (int)tol ||
       abs( (int)g-(int)data[ind+1] ) > (int)tol ||
       abs( (int)b-(int)data[ind+2] ) > (int)tol)
    return;

  //Set the color
  data[ind]   = f_r;
  data[ind+1] = f_g;
  data[ind+2] = f_b;
  (*area) ++;

  //Now recurse in 4 directions.
  seedFill(x+1, y, r, g, b, f_r, f_g, f_b, tol, area);
  seedFill(x-1, y, r, g, b, f_r, f_g, f_b, tol, area);
  seedFill(x, y+1, r, g, b, f_r, f_g, f_b, tol, area);
  seedFill(x, y-1, r, g, b, f_r, f_g, f_b, tol, area);
}


//OPENGL FUNCTIONS
//OPENGL FUNCTIONS
//OPENGL FUNCTIONS
//OPENGL FUNCTIONS
//OPENGL FUNCTIONS

void drawPointList(){
  int i;
  int *x, *y;
  int *seeds;

  x = cutter->plist.x;
  y = cutter->plist.y;
  seeds = cutter->plist.seed;

  glColor3f(1.0, 1.0, 0.0);
  glBegin(GL_POINTS);
    for (i=0; i<cutter->plist.num_points; i++){
      glVertex2i(x[i], cut_yres - y[i]);
    }
  glEnd();

  glPointSize(5.0);
  glColor3f(0.0, 1.0, 0.0);
  glBegin(GL_POINTS);
    for (i=0; i<cutter->plist.num_seeds; i++){
      glVertex2i(x[seeds[i]], cut_yres - y[seeds[i]]);
    }
  glEnd();
  glPointSize(1.0);

}

void drawWaveFront(){
  int j;
  GraphNode *scan_wave;

  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_POINTS);
    for (j=0; j<WAVE_SIZE; j++){
      for (scan_wave = cutter->wavefront[j];
           NULL != scan_wave;
           scan_wave = scan_wave->next){
        glVertex2i(scan_wave->x, cut_yres - scan_wave->y);
      }
    }
  glEnd();
}
/******************************************************************************
 * Purpose:  used by OpenGL to display the image.
 *****************************************************************************/
void display (void) {
  int wait_time;

  /*
  glMatrixMode (GL_PROJECTION );
    glLoadIdentity();
    glScalef(1.0, -1.0, 0.0); glMatrixMode (GL_MODELVIEW);
   */

  glRasterPos2i( 0, 0 );
  glDrawPixels(cut_xres, cut_yres, GL_RGB, GL_UNSIGNED_BYTE, cut_image->data);
  drawPointList();
  drawWaveFront();
  glFlush();//finish rendering it all and dump it out
  glutSwapBuffers();//since we're using the GLUT_DOUBLE

  wait_time = clock() + g_delay;
  while(wait_time > clock()) cutter->updateWaveFront();
  glutPostRedisplay();
}

/******************************************************************************
 * Purpose:  initialize the opengl stuff
 *****************************************************************************/
void init () {
  glClearColor (0.0, 0.0, 0.0, 0.0);
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity();
//  gluOrtho2D(0.0, cut_xres, cut_yres, 0.0);
  gluOrtho2D(0.0, cut_xres, 0.0, cut_yres);
//  glOrtho(0.0, xres, 0.0, yres, yres, -yres);
  glColor3f ( 1.0, 1.0, 1.0);
  glClear (GL_COLOR_BUFFER_BIT );//need to clear the buffer to do the z-buffer
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
  g_delay = (int)( (double)CLOCKS_PER_SEC/30.0);
}

/******************************************************************************
 * Purpose:  called by opengl to handle mouse events
 *****************************************************************************/
void mouseButton(int button, int state, int x, int y) {
  if (x<0) x = 0; if (x>cut_xres-1) x = cut_xres-1;
  if (y<0) y=0; if (y>cut_yres-1) y = cut_yres-1;
//  y = cut_yres - y;//switch the coordiates
  switch (button) {
    case GLUT_LEFT_BUTTON:
      if (state == GLUT_DOWN) {
        button_down = TRUE;
        /*
        cutter->plist.x[cutter->plist.num_points] = x;
        cutter->plist.y[cutter->plist.num_points] = y;
        cutter->plist.num_points ++;
        cutter->addSeedPoint();
        cutter->initWaveFront(x, y);
        cutter->updatePointList(x, y);
        //TEMPORARY
        cutter->updateWaveFront();
        */
//        cutter->dumpWaveFront();
//        exit(0);
      }
      if (state == GLUT_UP) {
        button_down = FALSE;
      }
      break;
      /*
      if (state == GLUT_UP) {
        button_down = FALSE;
        to.x = (double)x;
        to.y = (double)y;
        warp();
      }
      break;
      */

    case GLUT_RIGHT_BUTTON:
      break;
  }
}

/******************************************************************************
 * Purpose:  called by opengl to handle mouse events
 *****************************************************************************/
void mouseMotion(int x, int y) {
//  y = cut_yres - y;//switch the coordiates
  if (x<0) x = 0; if (x>cut_xres-1) x = cut_xres-1;
  if (y<0) y=0; if (y>cut_yres-1) y = cut_yres-1;

  cutter->updatePointList(x, y);
//  printf("Motion\n");
  /*
  if (button_down == TRUE) {
    y = yres - y;//switch the coordiates
    to.x = (double)x;
    to.y = (double)y;
    warp();
  }
  */
}

/******************************************************************************
 * Purpose:  called by opengl to handle keyboard events
 *****************************************************************************/
void keyboard(unsigned char key, int x, int y) {
  if (x<0) x = 0; if (x>cut_xres-1) x = cut_xres-1;
  if (y<0) y=0; if (y>cut_yres-1) y = cut_yres-1;

  switch ( key ) {
    case 'w':
      /*
      printf("warp the image\n");
      warp();
      */
      break;
    case 'a':
      if (!button_down) return;
      cutter->plist.x[cutter->plist.num_points] = x;
      cutter->plist.y[cutter->plist.num_points] = y;
      cutter->plist.num_points ++;
      cutter->addSeedPoint();
      cutter->initWaveFront(x, y);
      cutter->updateWaveFront();
      cutter->updatePointList(x, y);
      display();
      break;
    case 'c':
      if (!button_down) return;
      cutter->saveBoundary(x, y);
      break;
    case 'd':
      cutter->dumpCostGraph();
      break;
    case 's':
      cutter->removeSeedPoint();
      cutter->updateWaveFront();
      cutter->updatePointList(x, y);
      display();
      break;
    case 'p':
      cutter->dumpPaths();
      break;
    case 'q':
      exit (0);
    case 'h':
      printf(" KEYS:\n");
      printf(" 'a' - Add a seed point\n");
      printf(" 'c' - Close the loop selection and save out in a file \"bound.txt\"\n");
      printf(" 'd' - save out a cost graph image as \"cost_graph.pgm\"\n");
      printf(" 's' - Subtract a seed point\n");
      printf(" 'p' - Dump bad path info (should be nothing)\n");
      printf(" 'q' - EXIT\n");
      break;
    case 't':
//      ContourMaker *cont = new ContourMaker(cut_image);
//      delete cont;
      break;
  }
}





//FUNCTIONS
//FUNCTIONS
//FUNCTIONS
//FUNCTIONS
/*
 * Note   The Id of the shape is equvalent to the Red component value.
 */
void getColorFromID (int id, unsigned char *r,  unsigned char *g,  unsigned char *b ){
  *r = (unsigned char) ((77*id+67)%255);
  *g = (unsigned char)((id*21+90)%255);
  *b = (unsigned char)(id%255);//(unsigned char)((id*13+90)%255);
  /*
  c_r = 0;
  c_g = (unsigned char)(255.0 * .33333);
  c_b = (unsigned char)(255.0 * .66666);
  */
        /*
        c_r = (c_r + 41)%255;
        c_g = (unsigned char)((255.0 - (double)c_r)*0.33333);
        c_b = (unsigned char)((255.0 - (double)c_r)*0.66666);
        */
}

void getNeighbor8(int x, int y, int *nx, int *ny, int id){
  switch (id){
    case 0:
      *nx = x+1;
      *ny = y;
      break;
    case 1:
      *nx = x+1;
      *ny = y-1;
      break;
    case 2:
      *nx = x;
      *ny = y-1;
      break;
    case 3:
      *nx = x-1;
      *ny = y-1;
      break;
    case 4:
      *nx = x-1;
      *ny = y;
      break;
    case 5:
      *nx = x-1;
      *ny = y+1;
      break;
    case 6:
      *nx = x;
      *ny = y+1;
      break;
    case 7:
      *nx = x+1;
      *ny = y+1;
      break;
    default:
      printf("Error in Cutter::getNeighbor8() id: %d\n", id);
      break;
  }

}


void getNeighbor4(int x, int y, int *nx, int *ny, int id){
  switch (id){
    case 0:
      *nx = x+1;
      *ny = y;
      break;
    case 1:
      *nx = x;
      *ny = y-1;
      break;
    case 2:
      *nx = x-1;
      *ny = y;
      break;
    case 3:
      *nx = x;
      *ny = y+1;
      break;
    default:
      printf("Error in Cutter::getNeighbor4() id: %d\n", id);
      break;
  }

}

/*
 * Purpose: Convolves two signals and deposites their value in the third signal
 *          The ordering is to pass the second signal across the first.
 */
void convolve1D (double *s1, int size1,
                 double *s2, int size2,
                 double *s3, int *size3){
  int i, n;

  *size3 = size1 + size2 -1;
  
  for (n=0; n<*size3; n++){
    s3[n] = 0;
    for (i=0; i<size1; i++){ //first signal
      if (n-i >= 0 && n-i < size2)
        s3[n] += s2[n-i]*s1[i]; 
    }
  }
  
}

Image* sobelFilter (Image *img2){
  Image *img3;
  unsigned char *data2, *data3;
  int w1, h1, w2, h2, w3, h3;
  int i, j, n, m;
  double total;
  double *sobel;
//  int total_time;
//  clock_t total_time;

  img2->makeGray();
  w1 = 3;
  h1 = 3;
  w2 = img2->width;
  h2 = img2->height;
  w3 = w1 + w2 -1;
  h3 = h1 + h2 -1;
  
  //kernel = new double [w1*h1];
  img3 = new Image(w3, h3, 5);

  sobel = new double[9];


    /*
    sobel[6] = -1; sobel[7] = 0; sobel[8] = 1;
    sobel[3] = -2; sobel[4] = 0; sobel[5] = 2;
    sobel[0] = -1; sobel[1] = 0; sobel[2] = 1;
    */


    sobel[6] = -2; sobel[7] = -2; sobel[8] = 0;
    sobel[3] = -2; sobel[4] = 0; sobel[5] = 2;
    sobel[0] = 0; sobel[1] = 2; sobel[2] = 2;


    /*
    sobel[6] = -1; sobel[7] = -2; sobel[8] = -1;
    sobel[3] = 0; sobel[4] = 0; sobel[5] = 0;
    sobel[0] = 1; sobel[1] = 2; sobel[2] = 1;
    */

  data2 = img2->data;
  data3 = img3->data;

//  total_time = clock();
//  printf("clock: %d\n", (int)clock());
  for (i=0; i<w3; i++){
    for (j=0; j<h3; j++){
      total = 0.0;
      for (m=0; m<w1; m++){
        for (n=0; n<h1; n++){
          if ( (i-m) >= 0 &&
               (i-m) < w2 &&
               (j-n) >= 0 &&
               (j-n) < h2 ){
            total += sobel[m + n*w1] * (double)data2[i-m + (j-n)*w2];
          }
        }
      }
      total/=2.0;
      total+= 128;
      if (total > 255.0) total = 255.0;
      if (total < 0.0) total = 0.0;
      data3[i + j*w3] = (unsigned char) total;
    }
  }
//  for (i=0; i<1000000; i++);
//  printf("clock: %d\n", (int)clock());
//  total_time = clock() - total_time;
//  printf("Time to convolve sobel with image: %1.20f seconds.\n", (double)total_time/(double)CLOCKS_PER_SEC);
//  printf("Time to convolve sobel with image: %d seconds.\n", total_time/CLOCKS_PER_SEC);
  
  return img3;
}

/*
 * Purpose: Convolves two signals and deposites their value in the third signal
 *          The ordering is to pass the second signal across the first.
 */
Image* convolve2D (Image *img1, Image *img2){
  Image *img3;
  unsigned char *data1, *data2, *data3;
  int w1, h1, w2, h2, w3, h3;
  int i, j, n, m;
  int total_time;
  double total;
  double *kernel;

  img1->makeGray();
  img2->makeGray();
  w1 = img1->width;
  h1 = img1->height;
  w2 = img2->width;
  h2 = img2->height;
  w3 = w1 + w2 -1;
  h3 = h1 + h2 -1;
  
  kernel = new double [w1*h1];
  img3 = new Image(w3, h3, 5);

  data1 = img1->data;
  data2 = img2->data;
  data3 = img3->data;

  total = 0.0;
  for (i=0; i<w1*h1; i++)
    total += (double)data1[i];
  for (i=0; i<w1*h1; i++)
    kernel[i] = (double)(data1[i])/total;


  total_time = clock();
  for (i=0; i<w3; i++){
    for (j=0; j<h3; j++){
      total = 0.0;
      for (m=0; m<w1; m++){
        for (n=0; n<h1; n++){
          if ( (i-m) >= 0 &&
               (i-m) < w2 &&
               (j-n) >= 0 &&
               (j-n) < h2 ){
            total += kernel[m + n*w1] * (double)data2[i-m + (j-n)*w2];
          }
        }
      }
      if (total > 255.0) total = 255.0;
      data3[i + j*w3] = (unsigned char) total;
    }
  }
  total_time = clock() - total_time;

  printf("Time to convolve the 2 images: %f seconds.\n", (double)total_time/(double)CLOCKS_PER_SEC);
  
  
  return img3;
}

void enhanceEdge1D (double *s1, int size1,
                    double *s2, int size2,
                    double *s3, int *size3){
  double s4[MAX_SIGNAL_SIZE];
    int size4;
  double s5[MAX_SIGNAL_SIZE];
    int size5;
  double delta[MAX_SIGNAL_SIZE];
    double total = 0.0;
    int size_delta;
  int i;


    total = 0.0;
    size_delta = size2;
    for (i=0; i<size_delta; i++){
        delta[i] = 0.0;
        total += s2[i];
    }
    delta[size2/2] = 2.0;

    //normalize the gaussian
    for (i=0; i<size2; i++){
        s2[i]/=total;
    }
    //Make a delta function the size of the gausian to make the signal have the 
    //same shift.
    //convolve s1 and s2
    convolve1D (s1, size1, s2, size2, s4, &size4);
    convolve1D (s1, size1, delta, size_delta, s5, &size5);
  
    /*
    //clear it out.
    for(i=0; i<*size3; i++){
      s3[i] = 0.0;
    }
    */
  
   *size3 = size5;
    for(i=0; i<size5; i++){
        printf("i: %d s3:%f =  s5:%f - s4:%f\n",i, s3[i], s5[i], s4[i]);
      s3[i] = s5[i] - s4[i];
      //s3[i] = -s4[i];// - s4[i];
    }
}

/*
 * c1 is the image.
 * c2 is the filter.
 * c3 is  2*c1 - c2*c1;
 */
void enhanceEdge2D (Complex *c1, int size1,
                    Complex *c2, int size2,
                    Complex *c3, int *size3){
  int i;
  Complex m;

  *size3 = size1;
    
  for(i=0; i<size1*size1; i++){
    m = mult(c1[i], c2[i]);
    c3[i].r = 2*c1[i].r - m.r;
    c3[i].i = 2*c1[i].i - m.i;
  }
}

void medianFilter1D(int range, double *s1, int size1, double *s2, int *size2){
  int i, j, median;
  double *samples;
  
  median = range/2;
  samples = new double[range];

  *size2 = size1;
  for(i=0; i<*size2; i++)
    s2[i] = 0.0;

  for(i=median; i<size1-median; i++){
    for(j=-median; j<median; j++)
      samples[median+j] = s1[i+j];
    slowSort(samples, range);
    s2[i] = samples[median];
  }

  delete [] samples;
}

void medianFilter2D(int median, double *s1, int w, int h, double *s2){
  int i, j, k, l, range;
  double *samples;
  
  range = 2*median + 1;
  /*
  if (range%2 != 0) range ++;
  median = range/2;
  */
  samples = new double[range*range];


  for(i=0; i<w*h; i++)
    s2[i] = 0.0;

  for(i=median; i<w-median; i++){
    for(j=median; j<h-median; j++){

      //now get samples for the median filter
      for(k=-median; k<=median; k++)
        for(l=-median; l<=median; l++)
          samples[ k+median + (l+median)*range] = s1[ k+i + (l+j)*w] ;

      slowSort(samples, range*range);

      /*
      if (s1[i+j*w] > 0){
        for (q=0; q<range*range; q++){
          printf("sample[%d]: %f\n", q, samples[q]);
        }
      }
      */

      s2[i+j*w] = samples[(range*range-1)/2];
     // s2[i+j*w] = s1[i+j*w];

    }
  }

  delete [] samples;
}

void normalize(double *d, int size){
  int i;
  double total;

  total = 0.0;
  for (i=0; i<size; i++){
    total += d[i];
  }
  for (i=0; i<size; i++){
    d[i]/=total;
  }
}

void slowSort(double *a, int size){
  int i, j;
    double temp;
    /*
  printf("pre sort\n");
  for (i=0; i<size; i++){
    printf("a[%d]: %f\n",i , a[i]);
  }
  */
  for (i=0; i<size; i++){
    for (j=i+1; j<size; j++){
            //swap
      if (a[j] < a[i]){
        temp = a[i];
        a[i] = a[j];
        a[j] = temp;
      }
    }
  }
  /*
  printf("post sort\n");
  for (i=0; i<size; i++){
    printf("a[%d]: %f\n",i , a[i]);
  }
  */

}

void threshold(double *a, int size, double val, double bot, double top){
  int i;

  for(i=0; i<size; i++){
    if (bot < a[i] && a[i] < top){
      a[i] = val;
    }
  }
}

/*
 * finds the average for a bunch of pixels and thresh holds the pixel based
 * on this average
 */
void threshBlock2D(double *a, int w, int h, int range, double val, double bot, double top, double *b){
  int i, j, l, m;
  double ave, count;
  for(i=0; i<w; i++) for(j=0; j<h; j++) b[i +j*w] = 0.0;
//  for(i=w-range; i<w; i++) for(j=h-range; j<h; j++) b[i +j*w] = 0.0;

  for(i=range; i<w-range; i++){
    for(j=range; j<h-range; j++){
      /*
      count = 1.0;
      ave = a[i + j*w];
      */
      count = 0.0;
      ave = 0.0;//a[i + j*w];
      for(l=i-range; l<i+range; l++){
        for(m=j-range; m<j+range; m++){
          ave += a[l + m*w];
          count ++;
        }
      }

      ave /= count;
      if (bot < ave && ave < top){ b[i + j*w] = val; }
      else{ b[i + j*w] = a[i + j*w]; }
    }
  }
}

void FFT1D(double *f, int size, Complex *F){
  int i;
  int offset;
  int new_size;

  new_size = findPowerOf2(size);
  if (new_size != size){
    offset = (new_size-size)/2;
    for (i=size-1; i>=0; i--){
      f[i+offset] = f[i];
    }
    //pad with zeros
    for (i=new_size-1; i>=size; i--){
      f[i] = 0.0;
      f[i-size] = 0.0;
    }
  }

  //now assign values into complex array
  for (i=0; i<new_size; i++){
    F[i].r = f[i];
    F[i].i = 0.0;
  }
  FFT1Drec(F, new_size, new_size/2, 1);
}

/*
 * Note: size refers to the size of both dimensions
 */
void FFT2D(double *f, int size, Complex *F){
  int i, r, c;
  int N;
  Complex *X;
  N = findPowerOf2(size);
  if (N != size){
    printf(" Error, image is not a power of 2\n");
    return;
  }

  X = new Complex[N];
  //now assign values into complex array
 
  for (i=0; i<N*N; i++){
    F[i].r = f[i];
    F[i].i = 0.0;
  }

  //First the rows
  for (r=0; r<N; r++){
    for (i=0; i<N; i++){
      X[i].r = F[i + r*N].r; 
      X[i].i = F[i + r*N].i;
    }
    FFT1Drec(X, N, N/2, 1);
    for (i=0; i<N; i++){
      F[i + r*N].r = X[i].r;
      F[i + r*N].i = X[i].i;
    }
  }

  //now the columns
  for (c=0; c<N; c++){
    for (i=0; i<N; i++){
      X[i].r = F[c + i*N].r;
      X[i].i = F[c + i*N].i;
    }
    FFT1Drec(X, N, N/2, 1);
    for (i=0; i<N; i++){
      F[c + i*N].r = X[i].r;
      F[c + i*N].i = X[i].i;
    }
  }
  delete [] X;
}

/*
 * Note: size refers to the size of both dimensions
 */
void invFFT2D(Complex *F, int size, double *f){
  int i, r, c;
  int N;
  Complex *X;

  N = findPowerOf2(size);
  if (N != size){
    printf(" Error, image is not a power of 2\n");
    return;
  }

  X = new Complex[N];
  //now assign values into complex array
 

  //First the rows
  for (r=0; r<N; r++){
    for (i=0; i<N; i++){
      X[i].r = F[i + r*N].r; 
      X[i].i = F[i + r*N].i;
    }
    FFT1Drec(X, N, N/2, -1);
    for (i=0; i<N; i++){
      F[i + r*N].r = X[i].r;
      F[i + r*N].i = X[i].i;
    }
  }

  //now the columns
  for (c=0; c<N; c++){
    for (i=0; i<N; i++){
      X[i].r = F[c + i*N].r;
      X[i].i = F[c + i*N].i;
    }
    FFT1Drec(X, N, N/2, -1);
    for (i=0; i<N; i++){
      F[c + i*N].r = X[i].r;
      F[c + i*N].i = X[i].i;
    }
  }

  for (i=0; i<N*N; i++){
    f[i] = F[i].r;
  }


  delete [] X;
}

void FFT1Drec(Complex *F, int size, int mid, int dir){
  int i, new_size;
  double tempr, tempi;
  Complex he, e;
  Complex *G = new Complex[size/2];
  Complex *H = new Complex[size/2];
  double g_total;
  double h_total;

  if (size == 2){
    tempr = F[0].r + F[1].r;
    tempi = F[0].i + F[1].i;
    F[1].r = F[0].r - F[1].r;
    F[1].i = F[0].i - F[1].i;
    F[0].r = tempr;
    F[0].i = tempi;
    delete [] G;
    delete [] H;
    return;
  }
  else{
    h_total = g_total = 0.0;
    for(i=0, new_size=0; i<size;){
      G[new_size].r = F[i].r;
      G[new_size].i = F[i].i;
      g_total += G[new_size].getMag();
      i++;
      H[new_size].r = F[i].r;
      H[new_size].i = F[i].i;
      h_total += H[new_size].getMag();
      i++;
      new_size ++;
    }
    if (g_total > DBL_TOL)
      FFT1Drec(G, new_size, mid, dir);
    if (h_total > DBL_TOL)
      FFT1Drec(H, new_size, mid, dir);
    for(i=0; i<size; i++){
      /*
      e.r = cos( (TWO_PIE*(double)(i-mid))/(double)size ); 
      e.i = -dir*sin( (TWO_PIE*(double)(i-mid))/(double)size ); 
      */
      e.r = cos( (TWO_PIE*(double)i)/(double)size ); 
      e.i = -dir*sin( (TWO_PIE*(double)i)/(double)size ); 

      he = mult(e, H[i%new_size]);
      F[i].r = G[i%new_size].r + he.r;
      F[i].i = G[i%new_size].i + he.i;
    }
  }
  delete [] G;
  delete [] H;
}


void invFFT1D(Complex *F, int size, double *f){
  int i;
  int offset;
  int new_size;
  double invN;

  new_size = findPowerOf2(size);
  offset = (new_size-size)/2;
  for (i=size-1; i>=0; i--){
    F[i+offset].r = F[i].r;
    F[i+offset].i = F[i].i;
  }
  //pad with zeros
  for (i=new_size-1; i>=size; i--){
    F[i].r = 0.0;
    F[i-size].r = 0.0;
    F[i].i = 0.0;
    F[i-size].i = 0.0;
  }

  FFT1Drec(F, new_size, new_size/2, -1);
  //now assign values into complex array
  invN = 1.0/(double)new_size;
  for (i=0; i<new_size; i++){
    f[i] = invN*F[i].r;
  }
}

void FourierTransform1D(double *f, int size, Complex *F){
  int u, k, half;
  double N;

  N = (double) size;
  half = size/2;
  for (u=0; u<size; u++){
    F[u].r = 0.0;
    F[u].i = 0.0;
    for (k=0; k<size; k++){
      //F[u].r += f[k] * cos( (TWO_PIE*(double)(u)*(double)(k))/N ); 
      F[u].r += f[k] * cos( (TWO_PIE*(double)(u-half)*(double)(k-half))/N ); 
      F[u].i -= f[k] * sin( (TWO_PIE*(double)(u-half)*(double)(k-half))/N ); 
    }
  }
}

void invFourierTransform1D(Complex *F, int size, double *f){
  int u, k, half;
  double N, invN;
  Complex f_c[size];
  Complex ex;

  N = (double) size;
  invN = 1.0/N;
  half = size/2;
  for (k=0; k<size; k++){
    f_c[k].r = 0.0;
    f_c[k].i = 0.0;
    for (u=0; u<size; u++){
      ex.r = cos( (TWO_PIE*(double)(u-half)*(double)(k-half))/N );
      ex.i = sin( (TWO_PIE*(double)(u-half)*(double)(k-half))/N );
      ex = mult(ex, F[u]);
      f_c[k] = add(f_c[k], ex);
    }
  }

  for (k=0; k<size; k++){
    f[k] = invN*f_c[k].r;
  }
}

int findPowerOf2(int size){
  int newdim=1;
  int power =0;
  int tmp_old = 0, test = 0;

  tmp_old = test = size;

  while(size != 0){
    ++power;
    size >>=1;
  }

  newdim <<= power;

  if ( (test <<= 1) == newdim)
    return tmp_old;//the original value is already a power of two

  return newdim;
}


Complex add(Complex c1, Complex c2){
  Complex result;
  result.r = c1.r + c2.r;
  result.i = c1.i + c2.i;
  return result;
}

Complex sub(Complex c1, Complex c2){
  Complex result;
  result.r = c1.r - c2.r;
  result.i = c1.i - c2.i;
  return result;
}

Complex mult(Complex c1, Complex c2){
  Complex result;
  /*
  double phase;
  double mag;
  mag = c1.getMag() * c2.getMag();
  phase = c1.getPhase() + c2.getPhase();
  result.r = mag*cos(phase);
  result.i = mag*sin(phase);
  */

  result.r = c1.r*c2.r - c1.i*c2.i;
  result.i = c1.r*c2.i + c1.i*c2.r;
  return result;
}

Complex div(Complex c1, Complex c2){
  double mag1, magsq;
  Complex result;

  /*
  double phase;
  double mag;
  
  mag = c2.getMag();
  if (mag < DBL_TOL){
    result.r = 0.0;
    result.i = 0.0;
  }
  else{
    mag = c1.getMag() / c2.getMag();
    phase = c1.getPhase() - c2.getPhase();
    result.r = mag*cos(phase);
    result.i = mag*sin(phase);
  }
  */

  mag1 = c1.getMag();
  magsq = c2.getMag();

  if (magsq < DBL_TOL || mag1 < DBL_TOL){
    result.r = 0.0;
    result.i = 0.0;
  }
  else{
    magsq*=magsq;
    result.r = (c1.r*c2.r + c1.i*c2.i)/magsq;
    result.i = (-c1.r*c2.i + c1.i*c2.r)/magsq;
  }
  return result;
}

void mult(double *s1, int size1, double *s2, int size2, double *s3, int *size3){
  int half_min, i;
  int half1, half2, half3;
  half1 = size1/2;
  half2 = size2/2;

  if (size1 > size2){ half3 = half1; *size3 = size1; half_min = half2; }

  else{ half3 = half2; *size3 = size2; half_min = half1; }

  //clean out the signal
  for (i=0; i<*size3; i++){ s3[i] = 0.0; }

  //multiply the two
  for (i=0; i<half_min; i++){
    s3[half3 + i] = s1[half1 + i]*s2[half2 + i];
    s3[half3 - i] = s1[half1 - i]*s2[half2 - i];
  }

}

void add(double *s1, int size1, double *s2, int size2, double *s3, int *size3){
  int half_min, i;
  int half1, half2, half3;
  half1 = size1/2;
  half2 = size2/2;

  if (size1 > size2){
    half3 = half1; *size3 = size1; half_min = half2;
    for (i=0; i<*size3; i++){ s3[i] = s1[i]; }
  }

  else{
    half3 = half2; *size3 = size2; half_min = half1;
    for (i=0; i<*size3; i++){ s3[i] = s2[i]; }
  }


  //add the two
  for (i=0; i<half_min; i++){
    s3[half3 + i] = s1[half1 + i] + s2[half2 + i];
    s3[half3 - i] = s1[half1 - i] + s2[half2 - i];
  }

}

void mult(Complex *s1, int size1, Complex *s2, int size2, Complex *s3, int *size3){
  int half_min, i;
  int half1, half2, half3;
  half1 = size1/2;
  half2 = size2/2;

  if (size1 > size2){ half3 = half1; *size3 = size1; half_min = half2; }

  else{ half3 = half2; *size3 = size2; half_min = half1; }

  //clean out the signal
  for (i=0; i<*size3; i++){ s3[i].r = 0.0; s3[i].i = 0.0; }

  //multiply the two
  for (i=0; i<half_min; i++){
    s3[half3 + i] = mult(s1[half1 + i], s2[half2 + i]);
    s3[half3 - i] = mult(s1[half1 - i], s2[half2 - i]);
  }

}

/*
 * s3  = s1/s2
 */
void div(Complex *s1, int size1, Complex *s2, int size2, Complex *s3, int *size3){
  int half_min, i;
  int half1, half2, half3;
  half1 = size1/2;
  half2 = size2/2;

  if (size1 > size2){ half3 = half1; *size3 = size1; half_min = half2; }

  else{ half3 = half2; *size3 = size2; half_min = half1; }

  //clean out the signal
  for (i=0; i<*size3; i++){ s3[i].r = 0.0; s3[i].i = 0.0; }

  //multiply the two
  /*
  for (i=0; i<half_min; i++){
    s3[half3 + i] = div(s1[half1 + i], s2[half2 + i]);
    s3[half3 - i] = div(s1[half1 - i], s2[half2 - i]);
  }
  */
  for (i=0; i<size1; i++){
    s3[i] = div(s1[i], s2[i]);
  }
}

/*
 * Purpose: Get bounding boxes for connected components that have enough area.
 *          if the  is big enough, do a local threshold and union the
 *          results into a final image.
 */
int  boxThreshold(Image *img, Image *img_out){
  double thresh;
  unsigned char t;
  double *img_dbl, *img_dbl2;
  Image img_temp;
  Image img2;
  int w, h;
  int bot_x, bot_y, top_x, top_y;
  int range = 3;
  unsigned char num_shapes, i;
  int j;
  //char name[255];
  

  img_temp.copy(img);
  w = img->width;
  h = img->height;

  img_dbl = img->getDouble(0);
  img_dbl2 = new double[w*h];
  printf("DBG07w: %d h: %d\n", w, h);
  for (j=0; j<w*h; j++){
    img_dbl2[j] = 0.0;
  }

  thresh = getAverage(img_dbl, w*h, -1.0, 0);

  //make black
  threshold(img_dbl, w*h, 0, 0, 64);
  //make white
  threshold(img_dbl, w*h, 255, 255-64, 255);

  threshBlock2D(img_dbl, w, h, range, 255.0, thresh, 255.0, img_dbl2);
  threshold(img_dbl2, w*h, 0, 0, 250);

  img_temp.signalToImage2D(img_dbl2, w, h);
  img_temp.write("out_simple_thresh.pgm");

  //TEMP
  img2.copy(&img_temp);
  img2.erode(range);
  img2.erode(range);
  img2.dilate(range);
  img2.dilate(range);
  img2.write("out_simple_after_erode_dilate.pgm");
  num_shapes = img2.colorShapes(1, (w*h)>>9);
  img2.write("out_simple_thresh_color.pgm");

  num_shapes = img_temp.colorShapes(1, (w*h)>>9);
//  img_temp.write("out_simple_thresh_color.pgm");
//  num_shapes = 5;

  img_out->clear(0);
  //Now refine around each object
  for (i=1; i<=num_shapes; i++){
    img_temp.getItemBounds(i, &bot_x, &bot_y, &top_x, &top_y);
    bot_x -= (top_x- bot_x)/3;
    bot_y -= (top_y- bot_y)/3;
    top_x += (top_x- bot_x)/3;
    top_y += (top_y- bot_y)/3;
    if (top_x > img_temp.width) top_x = img_temp.width;
    if (top_y > img_temp.height) top_y = img_temp.height;
    if (bot_x < 0) bot_x = 0;
    if (bot_y < 0) bot_y = 0;
    t = img->getBoundAverage(bot_x, bot_y, top_x, top_y);
    img_out->unionThreshRegion(img, bot_x, bot_y, top_x, top_y, range, t);
    //img_out->write(name);
  }
  img_out->write("out_box_thresh.pgm");

  //Make back to black and white.
  img_out->threshold(2);

  /*
  delete [] img_dbl;
  img_dbl = img_out->getDouble(0);
  medianFilter2D(1, img_dbl, w, h, img_dbl2);
  img_out->signalToImage2D(img_dbl2, w, h); img_out->write("out_box_thresh_median.pgm");
  */

  img_out->erode(range);
  img_out->erode(range);
  img_out->write("out_after_erode.pgm");

  /*
  delete [] img_dbl;
  img_dbl = img_out->getDouble(0);
  medianFilter2D(1, img_dbl, w, h, img_dbl2);
  img_out->signalToImage2D(img_dbl2, w, h); img_out->write("out_box_thresh_median.pgm");
  */

  img_out->dilate(range);
  img_out->dilate(range);
  img_out->write("out_dilate.pgm");
  num_shapes = img_out->colorShapes(1, (w*h)>>9);
  img_out->write("out_after_color.pgm");

  delete [] img_dbl;
  delete [] img_dbl2;
  return num_shapes;
}

void copyDouble(double *dest, double *src, int size){
  int i;
  for (i=0; i<size; i++){
    dest[i] = src[i];
  }
}

void dilate(double *in, int w, int h, double *out, double *filter, int size, double tol){
  int i, j, l, m;
  int offset;

  offset = (size-1)/2;
  for(i=0; i<h*w; i++){
    out[i] = 0.0;
  }
  for(i=offset; i<w-offset; i++){
    for(j=offset; j<h-offset; j++){

      //If the pixel is set.
      if(in[i + j*w] > tol){
        for(l=0; l<size; l++){
          for(m=0; m<size; m++){
            if(filter[l + m*size] > tol)
              out[i-offset+l+(j-offset+m)*w] = filter[l + m*size];
          }
        }
      }


    }
  }
}

/*
 */
void erode(double *in, int w, int h, double *out, double *filter, int size, double tol){
  int i, j, l, m;
  int offset;
  int set;

  offset = (size-1)/2;
  for(i=0; i<h*w; i++){
    out[i] = 0.0;
  }
  for(i=offset; i<w-offset; i++){
    for(j=offset; j<h-offset; j++){
      set = 1;
      for(l=0; l<size; l++){
        for(m=0; m<size; m++){
          if(filter[l + m*size] > tol && in[i-offset+l+(j-offset+m)*w]<=tol){
              set = 0;
              m = l = size;//break out
          }
        }
      }
      //Now set the pixel in the outgoing image if needs be.
      if (1 == set)
        out[i + j*w] = in[i + j*w];
      else
        out[i + j*w] = 0.0;
    }
  }
}


/*
 * purpose: returns the average value of the all non zero pixels in the scene
 */
double getAverage(double *s, int size, double zero_intensity, double tol){
  int i;
  double total;
  double count;

  total = 0.0;
  count = 0.0;
  for (i=0; i<size; i++){
    if (s[i]+tol > zero_intensity && s[i]-tol < zero_intensity)
      continue;
    total += s[i];
    count ++;
  }

  return total/count;
}

/*
 * returns the percentage of this value (within a tol) in the array
 */
double getPercentValue(double *s, int size, double val, double tol){
  int i;
  double total;
  total = 0.0;
  for (i=0; i<size; i++){
    if ( s[i] + tol >= val && s[i] - tol <= val){
      total ++;
    }
  }
  return total/(double)size;
}

/*
 * get the difference between two arrays
 */
double getDiff(double *s1, double *s2, int size){
  double total;
  int i;

  total = 0.0;
  for (i=0; i<size; i++){
    total += fabs(s1[i] - s2[i]);
  }
  return total;
}

/*
 * get the average difference between two arrays
 */
double getAveDiff(double *s1, double *s2, int size){
  return getDiff(s1, s2, size)/(double)size;
}

void invert(Complex *s1, int size1){
  double magsq; 
  for (int i=0; i<size1; i++){
    magsq = s1[i].getMag()*s1[i].getMag();
    if (magsq < DBL_TOL){
      s1[i].r = 0.0;
      s1[i].i = 0.0; 
    }
    else{
      s1[i].r /= magsq;
      s1[i].i /= -magsq;
    }
  }
}

/*
 * Purpose: rotates the image the specified amount.
 * Note:    The value passed in is assumed to be in degrees
 */
void rotate2D(double *s, int n, double *d, double theta){
  int nx, ny;
  double x1, x2, y1, y2; 
  double sint, cost;
  double xc, yc, tx, ty;
  double sample1, sample2;
  int ix2, iy2;

  //convert to radians
  theta *= PIE/180.0;
  sint = sin(theta);
  cost = cos(theta);

  xc = (double)n/2.0;
  yc = (double)n/2.0;

  //To do the backwards mapping, we'll just take samples in the new image
  //and rotate back to the previous to get the value
  for (ny = 0; ny<n; ny++){
    for (nx = 0; nx<n; nx++){

      x1 = (double)nx;
      y1 = (double)ny;
      x2 = (x1-xc)*cost - (y1-yc)*sint + xc;
      y2 = (x1-xc)*sint + (y1-yc)*cost + yc;
      ix2 = (int)x2;
      iy2 = (int)y2;
      tx = x2 - (double)ix2;
      ty = y2 - (double)iy2;
      
      if (ix2+1<n && ix2>=0 && iy2+1<n && iy2>=0){
        sample1 = (1.0-tx)*s[ix2 + iy2*n] + tx*s[ix2+1 + iy2*n];
        sample2 = (1.0-tx)*s[ix2 + (iy2+1)*n] + tx*s[ix2+1 + (iy2+1)*n];
        d[nx + ny*n] = (1-ty)*sample1 + ty*sample2;
      }
      else{
        d[nx + ny*n] = 0.0;
      }

    }
  }
}

/*
 * Swap the mag and phase of the two
 */
void swapMagPhase(Complex *c1, Complex *c2, int size){
  int i;
  Complex c_temp;

  for (i=0; i<size; i++){
    c_temp.r = c1[i].r;
    c_temp.i = c1[i].i;

    if (c1[i].getMag() > DBL_TOL && c2[i].getMag() > DBL_TOL){
      c1[i].r = c1[i].getMag() * c2[i].r/c2[i].getMag();
      c1[i].i = c1[i].getMag() * c2[i].i/c2[i].getMag();
    }
    if ( c2[i].getMag() > DBL_TOL && c_temp.getMag() > DBL_TOL){
      c2[i].r = c2[i].getMag() * c_temp.r/c_temp.getMag();
      c2[i].i = c2[i].getMag() * c_temp.i/c_temp.getMag();
    }
  }
}

/*
 * Purpose: Go through the image, and find the offending frequency and remove it
 */
void removeInterference(Complex *D_Data, int size){
  int x, y, i;
  double ave, val;
  ave = 0.0;

  for (i=0; i<(size*size); i++){
    ave += D_Data[i].getMag()/(double)(size*size);
  }
  printf("ave: %f\n",ave);

  for (x=1; x<size-1; x++){
    for (y=1; y<size-1; y++){
      val = D_Data[x+y*size].getMag();
      val -= ave*ave*0.1;

      if (val >  D_Data[x+1+y*size].getMag() &&
          val >  D_Data[x-1+y*size].getMag() &&
          val >  D_Data[x+(y+1)*size].getMag() &&
          val >  D_Data[x+(y-1)*size].getMag() ){
        D_Data[x+y*size].r = 0.0;
        D_Data[x+y*size].i = 0.0;
      }
    }
  }
}

void createSinusoid(int size, double w, double *s, double mag, int flag){
  int i;
  double N;
  int half = size/2;

  N = (double) size;
  if (flag == SINE){
    for (i=0; i<size; i++){
      s[i] = mag*sin( (w * (double)(i-half))/N);
    }
  }
  else if (flag == COSINE){
    for (i=0; i<size; i++){
      s[i] = mag*cos( (w * (double)(i-half))/N);
    }
  }

}

void createSobel (int s, double *sobel){
  int i;
  int h = s/2;

  for (i=0; i<s*s; i++){
    sobel[i] = 0.0;
  }

  sobel[(h-1)+(h+1)*s]=-2; sobel[(h)+(h+1)*s]=-2; sobel[(h+1)+(h+1)*s]= 0;
  sobel[(h-1)+(h)*s]=  -2; sobel[(h)+(h)*s]=   0; sobel[(h+1)+(h)*s]=   2;
  sobel[(h-1)+(h-1)*s]= 0; sobel[(h)+(h-1)*s]= 2; sobel[(h+1)+(h-1)*s]= 2;

}

void createSinusoid2D(int size, double freq, double *s, double mag, int axis, int flag){
  int i, j;
  double N;
  double val = 0.0;

  N = (double) size;
  if (axis == X_AXIS){
    for (i=0; i<size; i++){
      if (SINE == flag) val = mag*sin( (freq * (double)i)/N);
      else if (COSINE == flag) val = mag*cos( (freq * (double)i)/N);
      for (j=0; j<size; j++){
        s[i + j*size] = val;
      }
    }
  }
  else if (axis == Y_AXIS){
    for (j=0; j<size; j++){
      if(SINE == flag) val = mag*sin( (freq * (double)j)/N);
      else if(COSINE == flag) val = mag*cos( (freq * (double)j)/N);
      for (i=0; i<size; i++){
        s[i + j*size] = val;
      }
    }
  }
}

/*
 * Purpose: Implements a butterworth filter.
 */
void butterFilter(Complex *d, int size, double cut_off, double n){
  int i;
  Complex c;
  c.i = 0;
  c.r = 0;
  cut_off *= cut_off;

  for (i=0; i<size/2; i++){
    c.r = 1.0/(1.0 + pow( ((double)i*i)/ cut_off, n) );
    d[i] = mult(d[i], c);
    d[size - i] = mult(d[size-i], c);
  }
}


void plot1D(double *a, int size, char *plot_name, char *filename){
  FILE   *fpt;
  char plotfname[255];
  char buff[255];
  char fname[255];
  char ticks[255];

  sprintf(ticks, "set xtics %d", size/20);

  printf("saving plots %s\n", filename);

  sprintf(plotfname, "plot_%s.dat", filename);
  sprintf(fname, "plot_real_%s.plot", filename);
  
  fpt = fopen( fname, "wb" );
  fprintf(fpt, "set title \"%s\"\n", plot_name);
  fprintf(fpt, "set xrange [0:%d]\n", size);
  fprintf(fpt, "%s\n", ticks);
  fprintf(fpt, "plot \"%s\" with lines\n", plotfname);
  fclose(fpt);

  fpt = fopen( plotfname, "wb" );
  for (int i=0; i<size; i++){
    fprintf(fpt, "%f\n", a[i]);
  }
  fclose(fpt);
  sprintf(buff, "gnuplot -persist %s\n", fname);
  system(buff);
}



void plotComplex1D(Complex *a, int size, char *plot_name, char *filename){
  FILE   *fpt, *fpt_data;
  char plotfname[255];
  char plot_title[255];
  char fname[255];
  char buff[255];
  char ticks[255];

  sprintf(ticks, "set xtics %d", size/20);
  
  printf("saving complex plots %s\n", filename);


  /*
  //Real plot
  sprintf(plot_title, "%s (Real part)", plot_name);
  sprintf(plotfname, "plot_real_%s.dat", filename);
  sprintf(fname, "plot_real_%s.plot", filename);
  fpt = fopen( fname, "wb" );
  fprintf(fpt, "set title \"%s\"\n", plot_title);
  fprintf(fpt, "set xrange [0:%d]\n", size);
  fprintf(fpt, "%s\n", ticks);
  fprintf(fpt, "plot \"%s\" with lines\n", plotfname);
  fclose(fpt);
  fpt_data = fopen( plotfname, "wb" );
  for (int i=0; i<size; i++){
    fprintf(fpt_data, "%f\n", a[i].r);
  }
  fclose(fpt_data);
  sprintf(buff, "gnuplot -persist %s\n", fname);
  system(buff);


  //Imaginary plot
  sprintf(plot_title, "%s (Imaginary part)", plot_name);
  sprintf(plotfname, "plot_img_%s.dat", filename);
  sprintf(fname, "plot_img_%s.plot", filename);
  fpt = fopen( fname, "wb" );
  fprintf(fpt, "set title \"%s\"\n", plot_title);
  fprintf(fpt, "set xrange [0:%d]\n", size);
  fprintf(fpt, "%s\n", ticks);
  fprintf(fpt, "plot \"%s\" with lines\n", plotfname);
  fclose(fpt);
  fpt_data = fopen( plotfname, "wb" );
  for (int i=0; i<size; i++){
    fprintf(fpt_data, "%f\n", a[i].i);
  }
  fclose(fpt_data);
  sprintf(buff, "gnuplot -persist %s\n", fname);
  system(buff);

  //Magnitude plot
  sprintf(plot_title, "%s (Magnitude)", plot_name);
  sprintf(plotfname, "plot_mag_%s.dat", filename);
  sprintf(fname, "plot_mag_%s.plot", filename);
  fpt = fopen( fname, "wb" );
  fprintf(fpt, "set title \"%s\"\n", plot_title);
  fprintf(fpt, "set xrange [0:%d]\n", size);
  fprintf(fpt, "%s\n", ticks);
  fprintf(fpt, "plot \"%s\" with lines\n", plotfname);
  fclose(fpt);
  fpt_data = fopen( plotfname, "wb" );
  for (int i=0; i<size; i++){
    fprintf(fpt_data, "%f\n", a[i].getMag());
  }
  fclose(fpt_data);
  sprintf(buff, "gnuplot -persist %s\n", fname);
  system(buff);

  //Phase plot
  sprintf(plot_title, "%s (Phase)", plot_name);
  sprintf(plotfname, "plot_phase_%s.dat", filename);
  sprintf(fname, "plot_phase_%s.plot", filename);
  fpt = fopen( fname, "wb" );
  fprintf(fpt, "set title \"%s\"\n", plot_title);
  fprintf(fpt, "set xrange [0:%d]\n", size);
  fprintf(fpt, "%s\n", ticks);
  fprintf(fpt, "plot \"%s\" with lines\n", plotfname);
  fclose(fpt);
  fpt_data = fopen( plotfname, "wb" );
  for (int i=0; i<size; i++){
    fprintf(fpt_data, "%f\n", a[i].getPhase());
  }
  fclose(fpt_data);
  sprintf(buff, "gnuplot -persist %s\n", fname);
  system(buff);
  */

  //Power plot
  sprintf(plot_title, "%s (Power Spectrum)", plot_name);
  sprintf(plotfname, "plot_power_%s.dat", filename);
  sprintf(fname, "plot_power_%s.plot", filename);
  fpt = fopen( fname, "wb" );
  fprintf(fpt, "set title \"%s\"\n", plot_title);
  fprintf(fpt, "set xrange [0:%d]\n", size);
  //fprintf(fpt, "set xtics 16\n");
  fprintf(fpt, "%s\n", ticks);
  fprintf(fpt, "plot \"%s\" with lines\n", plotfname);
  fclose(fpt);
  fpt_data = fopen( plotfname, "wb" );
  for (int i=0; i<size; i++){
    fprintf(fpt_data, "%f\n", a[i].getMag()*a[i].getMag() );
  }
  fclose(fpt_data);
  sprintf(buff, "gnuplot -persist %s\n", fname);
  system(buff);

}

void writeUB(double *s, int size, char *filename){
  int i;
  unsigned char uc;
  FILE *fptr;

  printf("saving ub file %s\n", filename);
  fptr = fopen(filename, "wb");
  for (i=0; i<size; i++){
    uc = (unsigned char)(s[i] + 128);
    fputc(uc, fptr);
  }
  fclose(fptr);
  /*
  if ( fwrite( uc, 1, (size_t)1, fptr ) !=
       (unsigned int)1 ){
    puts("fail to write");
    return FALSE;
  }
  */
}

/*
  else if (5 == type){
    fprintf( fpt, "P5\n" );
    //if ( HeaderInfo!=NULL ) fprintf( F, "# %s\n", HeaderInfo );
    //else fprintf( F, "# Grayscale Image\n" );
    fprintf( fpt, "%d %d\n%d\n", width, height, max );
    for ( i=0; i<height; i++ ) {
      save_line = &data[i*width];
      if ( fwrite( (void*)save_line, 1, (size_t)width, fpt ) !=
           (unsigned int)width ){
        puts("fail to write");
        return FALSE;
      }
    }
    fclose( fpt );
    return TRUE;
  }
  */

void filterFourier2D(Complex *c, int size, int begin, int end){
  int i,j;
  for (i=begin; i<end; i++)
    for (j=begin; j<end; j++){
      c[i + j*size].r = 0.0; 
      c[i + j*size].i = 0.0; 
    }
}


//MAIN
//MAIN
//MAIN
//MAIN
//MAIN
//MAIN
/******************************************************************************
 * Purpose:  main... The head hancho function.
 *****************************************************************************/
int main(int argc, char** argv) {
  Image image;
  double *img_dbl = NULL;
  double *img_dbl2 = NULL;

  if (argc < 3 || argc >4) {
    printf("  Error:\nIncorrect command line arguments.  The format is:\n");
    printf(">dig [-flag] [image.ppm or image.pgm] \n");
    printf("Example:\ndig -p1 parrots.ppm\n\n");
    printf("flags are as follows:\n");
    printf("-p2    :Get the difference of two images.\n");
    printf("-p3    :Segment out objects in the image using thresholding.\n");
    printf("-cont  :Find the boundaries of the shapes using contour following.\n");
    printf("-cut   :Use Cutter on the image (Intelligent Scissors).\n");
    exit (1);
  }

  if (strcmp( "-p3", argv[1]) == 0 ){
    Image orig_img, im2, info_image;
    int num_shapes;
    
    if ( !image.read(argv[2]) ){
      printf("Error in reading the image\n");
      exit (1);
    }

    image.makeGray();
    orig_img.copy(&image);
    im2.copy(&image);

    num_shapes = boxThreshold(&image, &im2);


    info_image.copy(&im2);
    info_image.createInfoImage(num_shapes);
    info_image.write("out_info.ppm");
    orig_img.mask(&im2);
    orig_img.write("out_segmented.pgm");
  }

  else if (strcmp( "-p2", argv[1]) == 0 ){
    Image image2;
    if ( !image.read(argv[2]) ){
      printf("Error in reading the image\n");
      exit (1);
    }
    if ( !image2.read(argv[3]) ){
      printf("Error in reading the image\n");
      exit (1);
    }
    image.makeGray();
    image2.makeGray();

    image.diff(&image2);
    image.write("out_diff.pgm");
  }


  else if (strcmp( "-cont", argv[1]) == 0 ){
    ContourMaker *cont = NULL;
    if ( !image.read(argv[2]) ){
      printf("Error in reading the image\n");
      exit (1);
    }
    cont = new ContourMaker(&image);


    /*
    //TEMP
    int size, i;
    PointList pl2;
    FILE *fp;
    fp = fopen(argv[3], "r");
    if (fp == NULL) { printf("Could not open file %s\n", argv[3]); exit (1); }
    fscanf(fp,"%d", &size);
    pl2.num_points = size;
    for (i=0; i<size; i++){
      fscanf(fp,"%d", &pl2.x[i]);
      fscanf(fp,"%d", &pl2.y[i]);
    }
    fclose(fp);
    image.clear(0);
    cont->drawBoundaries(&image, &pl2);
    image.write("out_myst.ppm");
    //END TEMP
    */


    delete cont;
  }

  else if (strcmp( "-match", argv[1]) == 0 ){
    FILE *fp; 
    int i, size;
    double dist;
    PointList pl1, pl2;

    fp = fopen(argv[2], "r");
    if (fp == NULL) { printf("Could not open file %s\n", argv[2]); exit (1); }
    fscanf(fp,"%d", &size);
    pl1.num_points = size;
    for (i=0; i<size; i++){
      fscanf(fp,"%d", &pl1.x[i]);
      fscanf(fp,"%d", &pl1.y[i]);
    }
    fclose(fp);


    fp = fopen(argv[3], "r");
    if (fp == NULL) { printf("Could not open file %s\n", argv[3]); exit (1); }
    fscanf(fp,"%d", &size);
    pl2.num_points = size;
    for (i=0; i<size; i++){
      fscanf(fp,"%d", &pl2.x[i]);
      fscanf(fp,"%d", &pl2.y[i]);
    }
    fclose(fp);

    pl1.createShapeRep();
    pl2.createShapeRep();
    dist = pl1.getDist(&pl2);
    pl1.writeShapeRep("out_boundary");
    pl2.writeShapeRep("out_bound_shape");
    printf("dist between pl1 & pl2: %f\n", dist);
    /*
    for (i=0; i<size; i++){
      printf("%d ", pl1.x[i]);
      printf("%d\n", pl1.y[i]);
    }
    */
    //size1 = 0; EOF != fscanf(fp,"%lf", &signal1[size1]); size1++)
    //    ;
  }

  else if (strcmp( "-cut", argv[1]) == 0 ){
    char name[255];
    cut_image = new Image();
    if ( !cut_image->read(argv[2]) ){
      printf("Error in reading image %s\n", argv[2]);
      return 1;
    }
    image.copy(cut_image);
    cut_xres = cut_image->width;
    cut_yres = cut_image->height;

    //FOR GL RENDERING
    cut_image->flip();
    cut_image->makeColor();
    cutter = new Cutter(&image);
//    cutter->dumpCostGraph();

    glutInit(&argc, argv);
      //GLUT_DOUBLE used for double buffering
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE );
    glutInitWindowSize (cut_xres, cut_yres);
    glutInitWindowPosition (0, 0);
    strcpy(name, argv[0]);//name of the program
    strcat(name, " - Intelligent Scissors");
    glutCreateWindow (name);
    init();
      // The names of the functions that you use for displaying and events go
      // as the parameters of this functions below
    glutDisplayFunc(display);
    glutMouseFunc(mouseButton);
    glutMotionFunc(mouseMotion);
    glutKeyboardFunc(keyboard);
    glutMainLoop();
  }

  else{
    printf("Flag '%s' not supported\n", argv[1]);
  }
  if (NULL != img_dbl) delete [] img_dbl;
  if (NULL != img_dbl2) delete [] img_dbl2;

  return 0;
}


/*
 * ADDITIONAL CODE FOR PROJECTS. CODE NOT WRITTEN BY MIKE SMITH
 *
 *
 *
 */
/*==================================================*\
 * Code framework for BYU CS 450 Assignment 4, audio part.
 *
 * This source code is covered under the GNU Public License, Version 2
 * ( see http://www.gnu.org/ ).
\*====================================================*/


/* Write a long out to a stream in little-endian order */
void fwrite_long(unsigned long number, FILE *stream) {
  fputc((number & 0xff), stream);            fputc((number & 0xff00) >> 8, stream);
  fputc((number & 0xff0000) >> 16, stream);  fputc((number & 0xff000000) >> 24, stream);
}

/* Write a short out to a stream in little-endian order */
void fwrite_short(unsigned short number, FILE *stream) {
  fputc((number & 0x00ff), stream);
  fputc((number & 0xff00) >> 8, stream);
}


/* Write a WAV sound file (1-channel, 16-bit only).
 * Works on both big/little endian machines.
 */

void write_wav(char *filename, double *sample, unsigned long sample_len, unsigned long freq) {
  FILE *out = fopen(filename, "wb");
  double min, max, offset, scale;
  unsigned int i;

  /* Check output file opened OK */
  if (!out) {
    fprintf(stderr, "\nCould not open .wav file\n\n");
    exit(1);
  }

  /* Write the .wav header */
  fprintf(out, "RIFF");
  fwrite_long(2 * sample_len + 36, out);
  fprintf(out, "WAVEfmt ");
  fputc(16, out);  fputc(0, out);  fputc(0, out);   fputc(0, out);
  fputc(1, out);   fputc(0, out);  fputc(1, out);   fputc(0, out);
  fwrite_long(freq, out);  fwrite_long(freq * 2, out);
  fputc(2, out);   fputc(0, out);  fputc(16, out);  fputc(0, out);
  fprintf(out, "data");
  fwrite_long(sample_len * 2, out);
  
  /* Find the range of values in the sample */
  min = max = sample[0];
  for (i = 1; i < sample_len; i++) {
    if (sample[i] > max)  max = sample[i];
    if (sample[i] < min)  min = sample[i];
  }
  if (min == max) {
    max = 1.0;
    min = -1.0;    /* Avoid division by zero */
  }
  offset = (max + min) / 2.0;
  scale = ((double) SHRT_MAX * 2.0) / (max - min);

  /* Write the scaled samples to the file */
  for (i = 0; i < sample_len; i++) {
    /* Center the sample about 0, and scale it so its maximum value is +/- SHRT_MAX */
    short sample_val = (short)((sample[i] - offset) * scale +.5);
    // Uncomment the next line to output the first 250 sample values for plotting:
    //if (i < 250) printf("%d\n", (int) sample_val);
    fwrite_short((unsigned short) sample_val, out);  /* Convert to unsigned short for output */
  }

  fclose(out);
}


/*-----------------------------------------------------------------------*\
 * Create a sine wave sound sample with a fraction of random noise
\*----------------------------------------------------------*/

double *create_wav(int sample_len, double freq_scale, double noise_frac) {
  double *sample = (double *) malloc(sample_len * sizeof(double));
  int i;

  /* Create the sample */
  for (i = 0; i < sample_len; i++) {
    double a = (double) i * freq_scale;
    double s = sin(a);
    double r = noise_frac * (2.0 * (double) random() / ((double) RAND_MAX + 1.0) - 1.0);
    sample[i] = s + r;
  }
  return sample;
}

//EOF
